/**
 *	\file gre/3dclip.h
 *	\brief Clipping against unit cube [-1.0, 1.0] in homogenius coordinates
 *
 * \if NODOC
 * $Id: 3dclip.h_v 1.6 2013/02/15 20:54:32 mju Exp $
 *
 * $Log: 3dclip.h_v $
 * Revision 1.6  2013/02/15 20:54:32  mju
 * Fix dox hdr.
 *
 * Revision 1.5  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/08/14 19:51:26  vdronov
 * added docs
 *
 * Revision 1.3  2003/07/30 13:31:19  mju
 * Add inclusion guards and ignore private section.
 *
 * Revision 1.2  2003/03/12 22:42:06  vdronov
 * added ClipPoint
 *
 * Revision 1.1  2003/03/10 15:05:04  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_3DCLIP_H
#define	INC_GRE_3DCLIP_H

#ifndef	INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

namespace GRE {

//! CLIPPING3D class.
//!
//!	This class is designed to provide static clipping methods against unit cube {-1.0, 1.0] 
//! for each coordinate in homogenius coordinates.
class CLIPPING3D {
	public:

		enum PLANE {
			PLANE_X_P_W = 0,
			PLANE_X_M_W = 1,
			PLANE_Y_P_W = 2,
			PLANE_Y_M_W = 3,
			PLANE_Z_P_W = 4,
			PLANE_Z_M_W = 5,
			PLANE_Count = 6
			};

		//! Clip point
		//!
		//! @return true if point inside unit cube
		static bool ClipPoint	(
			DPOINT3DH& point
			);

		//! Clip segment with endpoints point1 and point2. If segment intersects unit cube than
		//! point1 and point2 contains endpoints of intersection in return.
		//!
		//! @return true if segment intersects unit cube 
		static bool ClipSegment	(
			DPOINT3DH& point1,
			DPOINT3DH& point2
			);

		//! Clip plane polygon input. If polygon intersects unit cube than
		//! output contains polygon of intersection in return.

		//!
		//! @return true if polygon intersects unit cube 
		static bool ClipPolygon (
			const DOUBLE_ARRAY<DPOINT3DH>& input,
			DOUBLE_ARRAY<DPOINT3DH>& output
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static bool Check (
			const double w1,
			const double w2,
			double& tmin,
			double& tmax
			);

		static bool ClipToPlane (
			const PLANE plane,
			const DOUBLE_ARRAY<DPOINT3DH>& input,
			DOUBLE_ARRAY<DPOINT3DH>& output
			);

		static bool CheckAbove (
			const double value
			) { return (value > 0.0); }

		static bool AbovePlane	(
			const PLANE plane,
			const DPOINT3DH& point
			);

		//! Assuming an intersection point exists always (for efficency)
		static DPOINT3DH IntersectPlane	(
			const PLANE plane,
			const DPOINT3DH& point1,
			const DPOINT3DH& point2
			);

		static bool Intersect (
			const double w1,
			const double w2,
			double& t
			);

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

};		//! End of namespace

#endif

