/**
 * \file gre/uiformviewsettings.h
 * \brief GRE FORM_VIEW::SETTINGS class definition
 *
 * \if NODOC
 * $Id: uiformviewsettings.h_v 1.13 2011/11/28 17:13:04 mju Exp $
 *
 * $Log: uiformviewsettings.h_v $
 * Revision 1.13  2011/11/28 17:13:04  mju
 * Fix dox dependency.
 *
 * Revision 1.12  2011/07/11 19:56:33  bcolombe
 * Add zoom panel
 *
 * Revision 1.11  2011/05/17 20:39:34  mju
 * Add getShowSidebar.
 *
 * Revision 1.10  2011/02/21 21:55:40  bcolombe
 * Add settings for places tool
 *
 * Revision 1.9  2010/03/04 22:45:21  mju
 * Add sidebar settings.
 *
 * Revision 1.8  2010/02/22 17:25:10  bcolombe
 * Added magnifier settings
 *
 * Revision 1.7  2008/08/28 13:22:02  mju
 * Fix doxygen warning.
 *
 * Revision 1.6  2008/04/04 20:49:12  mju
 * In setCanvas/Legend/Locator size, check that at least minimal size.
 *
 * Revision 1.5  2008/03/04 16:04:35  scowan
 * Added necessary c opy ctor and assign operator to settings class.
 *
 * Revision 1.4  2008/02/22 22:38:05  mju
 * SUpport saving layout settings.
 *
 * Revision 1.3  2008/01/10 16:05:28  mju
 * Add getStatusPosShowAny.
 *
 * Revision 1.2  2007/12/28 18:54:31  mju
 * Redesign so statuspos is class and accessed by index.
 *
 * Revision 1.1  2007/12/03 16:52:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_UIFORMVIEWSETTINGS_H
#define	INC_GRE_UIFORMVIEWSETTINGS_H

#ifndef	INC_GRE_UIFORMVIEW_H
	#include <gre/uiformview.h>
#endif

#ifndef	INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

namespace GRE {
namespace UI {

//===================================================================================================================

//! Constainer for savable settings for GRE::UI::FORM_VIEW.
class FORM_VIEW::SETTINGS : public SERIALIZABLE {
	public:

		//! Settings for statusbar position reports.
		class STATUSPOS {
			public:
				static const SERIALIZERITEM* SerialGetItemDef ();
				STATUSPOS ();
				~STATUSPOS ();
				const SPATREF::COORDREFSYS& GetCRS () const { return (m_CoordRefSys); }
				DMSTYPE GetLatLonFormat () const { return (m_LatLonFormat); }
				bool GetShow () const { return (m_Show); }
				void IniRead (INIHANDLE handle, const char *inigroup, const char *inikeybase);
				void IniWrite (INIHANDLE handle, const char *inigroup, const char *inikeybase) const;
				void SetCRS (const SPATREF::COORDREFSYS& CoordRefSys) { m_CoordRefSys = CoordRefSys; }
				void SetLatLonFormat (DMSTYPE LatLonFormat) { m_LatLonFormat = LatLonFormat; }
				void SetShow (bool Show) { m_Show = Show; }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				SPATREF::COORDREFSYS m_CoordRefSys;
				DMSTYPE m_LatLonFormat;
				bool m_Show;
				#endif
			};

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Default constructor.
		SETTINGS ();

		//! Copy constructor.
		SETTINGS (
			const SETTINGS& rhs
			);

		//! Destructor.
		virtual ~SETTINGS ();

		//! Assignment operator.
		SETTINGS& operator= (
			const SETTINGS& rhs
			);

		//! Get canvas height.
		//! @return Height in pixels.
		INT32 GetCanvasHeight (
			) const { return (m_CanvasHeight); }

		//! Get canvas width.
		//! @return Width in pixels.
		INT32 GetCanvasWidth (
			) const { return (m_CanvasWidth); }

		//! Get zoom pane height.
		//! @return Height in pixels.
		INT16 GetZoomHeight (
			) const { return (m_ZoomHeight); }

		//! Get places height.
		//! @return Height in pixels.
		INT16 GetPlacesHeight (
			) const { return (m_PlacesHeight); }

		//! Get locator height.
		//! @return Height in pixels.
		INT16 GetLocatorHeight (
			) const { return (m_LocatorHeight); }

		//! Get magnifier height.
		//! @return Height in pixels.
		INT16 GetMagnifierHeight (
			) const { return (m_MagnifierHeight); }

		//! Get magnifier zoom
		INT16 GetMagnifierZoom (
			) const { return (m_MagnifierZoom); }

		//! Get whether to show legend.
		bool GetShowLegend (
			) const { return (m_ShowLegend); }

		//! Get whether to show locator.
		bool GetShowLocator (
			) const { return (m_ShowLocator); }

		//! Get whether to show magnifier.
		bool GetShowMagnifier (
			) const { return (m_ShowMagnifier); }

		//! Get whether to show zoom pane.
		bool GetShowZoom (
			) const { return (m_ShowZoom); }

		//! Get whether to show places.
		bool GetShowPlaces (
			) const { return (m_ShowPlaces); }

		//! Get whether to show sidebar.
		bool GetShowSidebar (
			) const { return (m_ShowSidebar); }

		//! Get sidebar width.
		//! @return Width in pixels.
		INT16 GetSidebarWidth (
			) const { return (m_SidebarWidth); }

		//! Get status position settings.
		const STATUSPOS& GetStatusPos (
			unsigned idx								//! Report index (0 or 1)
			) const { return (m_StatusPos[idx]); }

		//! Determine if any position report shown.
		bool GetStatusPosShowAny (
			) const { return (m_StatusPos[0].GetShow() || m_StatusPos[1].GetShow()); }

		//! Read settings from INI file.
		void IniRead (
			INIHANDLE handle,							//!< INI handle, 0 for default
			const char *inigroup,					//!< INI group, 0 for default
			const char *ViewTypeName = 0			//!< View "type name", 0 for generic
			);

		//! Write all settings to INI file.
		void IniWrite (
			INIHANDLE handle = 0,					//!< INI handle, 0 for default
			const char *inigroup = 0				//!< INI group, 0 to use one specified in IniRead
			) const;

		//! Write layout (control arrangement) settings to INI file.
		void IniWriteLayout (
			INIHANDLE handle = 0,					//!< INI handle, 0 for default
			const char *inigroup = 0				//!< INI group, 0 to use one specified in IniRead
			) const;

		//! Set canvas dimensions.
		void SetCanvasDimensions (
			INT32 width,								//!< Width in pixels
			INT32 height								//!< Height in pixels
			) { if (width > 10 && height > 10) { m_CanvasWidth = width; m_CanvasHeight = height; } }

		//! Set canvas height.
		void SetCanvasHeight (
			INT32 height								//!< Height in pixels
			) { if (height > 10) m_CanvasHeight = height; }

		//! Set canvas width.
		void SetCanvasWidth (
			INT32 width									//!< Width in pixels
			) { if (width > 10) m_CanvasWidth = width; }

		//! Set zoom pane height.
		void SetZoomHeight (
			INT16 height								//!< Height in pixels
			) { if (height > 10) m_ZoomHeight = height; }

		//! Set places height.
		void SetPlacesHeight (
			INT16 height								//!< Height in pixels
			) { if (height > 10) m_PlacesHeight = height; }

		//! Set locator height.
		void SetLocatorHeight (
			INT16 height								//!< Height in pixels
			) { if (height > 10) m_LocatorHeight = height; }

		//! Set magnifier height.
		void SetMagnifierHeight (
			INT16 height								//!< Height in pixels
			) { if (height > 10) m_MagnifierHeight = height; }

		//! Set magnifier zoom.
		void SetMagnifierZoom (
			INT16 zoom								//!< Zoom (2x, 4x, 8x, 16x)
			) { m_MagnifierZoom = zoom; }

		//! Set whether to show legend.
		void SetShowLegend (
			bool show
			) { m_ShowLegend = show; }

		//! Set whether to show zoom pane.
		void SetShowZoom (
			bool show
			) { m_ShowZoom = show; }

		//! Set whether to show places.
		void SetShowPlaces (
			bool show
			) { m_ShowPlaces = show; }

		//! Set whether to show locator.
		void SetShowLocator (
			bool show
			) { m_ShowLocator = show; }

		//! Set whether to show magnifier.
		void SetShowMagnifier (
			bool show
			) { m_ShowMagnifier = show; }

		//! Set whether to show sidebar.
		void SetShowSidebar (
			bool show
			) { m_ShowSidebar = show; }

		//! Set sidebar width.
		void SetSidebarWidth (
			INT16 width									//!< Width in pixels
			) { if (width > 10) m_SidebarWidth = width; }

		//! Set all position report values.
		void SetStatusPos (
			unsigned idx,								//!< Report index (0 or 1)
			const STATUSPOS& StatusPos
			) { m_StatusPos[idx] = StatusPos; }

		//! Set position report CRS.
		void SetStatusPosCRS (
			unsigned idx,								//!< Report index (0 or 1)
			const SPATREF::COORDREFSYS& CoordRefSys
			) { m_StatusPos[idx].SetCRS(CoordRefSys); }

		//! Set geographic position report format.
		void SetStatusPosLatLonFormat (
			unsigned idx,								//!< Report index (0 or 1)
			DMSTYPE LatLonFormat
			) { m_StatusPos[idx].SetLatLonFormat(LatLonFormat); }

		//! Set whether to show position indicator.
		void SetStatusPosShow (
			unsigned idx,								//!< Report index (0 or 1)
			bool Show
			) { m_StatusPos[idx].SetShow(Show); }

		//! Update values from form.
		//! Typically done just prior to saving.
		void UpdateFromForm (
			UI::FORM_VIEW& form
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		char *m_IniGroupRead;
		char *m_ViewTypeName;
		STATUSPOS m_StatusPos[2];
		INT32 m_CanvasWidth;
		INT32 m_CanvasHeight;
		INT16 m_SidebarWidth;
		INT16 m_ZoomHeight;
		INT16 m_PlacesHeight;
		INT16 m_LocatorHeight;
		INT16 m_MagnifierHeight;
		INT16 m_MagnifierZoom;
		bool m_ShowSidebar;
		bool m_ShowLegend;
		bool m_ShowZoom;
		bool m_ShowPlaces;
		bool m_ShowLocator;
		bool m_ShowMagnifier;

		//! Overrides from SERIALIZABLE.
		virtual ERRVALUE SerialRead (SERIALIZER& serializer);
		virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIFORMVIEWSETTINGS_H
