/** 
 * \file mgui/formpassword.h 
 * \brief Definitions for MGUI::FORM_PASSWORD classes.
 *
 * \if NODOC
 * $Id: formpassword.h_v 1.9 2011/11/29 22:29:34 mju Exp $
 *
 * $Log: formpassword.h_v $
 * Revision 1.9  2011/11/29 22:29:34  mju
 * Remove redundant hdrs.
 *
 * Revision 1.8  2011/09/01 17:53:41  mju
 * Exclude macro from docx.
 *
 * Revision 1.7  2006/07/11 14:31:26  dwilliss
 * changed SetFocusToName to not be inline.  Gets a bit more complicated if
 * no name field to set focus on.
 *
 * Revision 1.6  2006/07/11 13:53:55  dwilliss
 * added flag for no user name
 *
 * Revision 1.4  2006/04/21 19:33:11  scowan
 *  Added more checks and uses new hasValue method.
 *
 * Revision 1.1  2006/04/12 22:56:40  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMPASSWORD_H
#define	INC_MGUI_FORMPASSWORD_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace MGUI {

//===================================================================================================================
//! User Name and Password entry form with optional label description and validate password fields
class FORM_PASSWORD : public MGUI::FORM_COMPOSITE {
	public:
		enum FLAGS {
			FLAG_Default = 				0x0000,
			FLAG_ValidatePassword =		0x0001,			//!< Adds extra field to re-enter password to validate against original password
			FLAG_RememberPassword =		0x0002,			//!< Adds toggle button "Remember this password for future use".  Cannot be used with FLAG_ValidatePassword.
			FLAG_NoUserName =		      0x0004,			//!< No user name prompt, just a password
			};
	
		//! Constructor.
		FORM_PASSWORD (
			);

		//! Destructor.
		virtual ~FORM_PASSWORD (
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& Description,				//!< String to be placed above User Name and Password, empty if none wanted.  Eg: HTTP Realm string
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Determine if the user wants us to remember the password for future use.
		//! It is up to the calling code to 'remember' the password.
		//! This method can only return true if the FLAG_RememberPassword is used.
		//! @return 'True' to remember the password, 'false' if not.
		bool DoRememberPassword (
			) const;

		//! Get HTTP Basic Authentication string
		//! Does not include: "Authorization: Basic" part
		void GetBasicAuthString (
			MISTRING& AuthString
			);

		//! Get password entered
		//! If you make a copy of the password, you need to make sure to clean up properly
		const MISTRING& GetPassword (
			);

		//! Get user name entered
		const MISTRING& GetUserName (
			);

		//! Set delegate to call to get notified on any descriptor changes
		void SetDelegateOnComplete (
			FastDelegate<void(bool)> delegate
			) { m_DelegateOnComplete = delegate; }

		//! Set delegate to call to validate user name.
		//! This delegate is called only if the FLAG_ValidatePassword is used.
		//! This delegate can be used to enforce a unique user name 
		void SetDelegateOnValidateUserName (
			FastDelegate<bool(const MISTRING&)> delegate
			) { m_DelegateOnValidateUserName = delegate; }

		//! Sets the focus to the name prompt
		//! Note: if the form was created with FLAG_NoUserName, this will 
		//!  actually set the focus to the password form.
		void SetFocusToName (
			);

		//! Set the UserName to a value.
		void SetUserName (
			const MISTRING& UserName
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LAYOUT_PANE m_VertPane;
		CTRL_LABEL m_Description;
		FORM_EDIT_STRING m_NameForm;
		FORM_EDIT_STRING m_PasswordForm;
		CTRL_LABEL m_ValidateLabel;
		FORM_EDIT_STRING m_ValidateForm;
		CTRL_TOGGLEBUTTON m_RememberPassword;
		
		MISTRING m_UserName;
		FastDelegate<void(bool)> m_DelegateOnComplete;
		FastDelegate<bool(const MISTRING&)> m_DelegateOnValidateUserName;
	
		void OnUserEditChange ();
		void OnUserValidate ();
		void OnValidateChangeValue ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_PASSWORD::FLAGS)
#endif


class DLG_PASSWORD : public DLGSHELL {
	public:
		//! Constructor.
		DLG_PASSWORD (
			const MISTRING& title,						//!< Title string, can use TEXTID also.  If empty uses TEXTID_Title_UserNameAndPassword
			const MISTRING& Description,				//!< String to be placed above User Name and Password, Eg: HTTP Realm string, empty if none wanted
			HELPID HelpID = HELPID__None,				//!< Help ID
			FORM_PASSWORD::FLAGS flags = FORM_PASSWORD::FLAG_Default
			);

		//! Destructor.
		virtual ~DLG_PASSWORD (
			);

		//! Determine if the user wants us to remember the password for future use.
		//! It is up to the calling code to 'remember' the password.
		//! This method can only return true if the FLAG_RememberPassword is used.
		//! @return 'True' to remember the password, 'false' if not.
		bool DoRememberPassword (
			) const { return (m_FormPassword.DoRememberPassword()); }
			
		//! Get HTTP Basic Authentication string
		//! Does not include: "Authorization: Basic" part
		void GetBasicAuthString (
			MISTRING& AuthString
			) { m_FormPassword.GetBasicAuthString(AuthString); }

		//! Get user name entered
		const MISTRING& GetUserName (
			) { return (m_FormPassword.GetUserName()); }

		//! Get password entered
		//! If you make a copy of the password, you need to make sure to clean up properly
		const MISTRING& GetPassword (
			) { return (m_FormPassword.GetPassword()); }

		//! Set delegate to call to validate user name.
		//! This delegate is called only if the FLAG_ValidatePassword is used.
		//! This delegate can be used to enforce a unique user name 
		void SetDelegateOnValidateUserName (
			FastDelegate<bool(const MISTRING&)> delegate
			) { m_FormPassword.SetDelegateOnValidateUserName(delegate); }

		void SetUserName (
			const MISTRING& UserName
			) { m_FormPassword.SetUserName(UserName); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_PASSWORD m_FormPassword;
		FORM_PASSWORD::FLAGS m_flags;
		MISTRING m_Description;
		
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOpen ();

		void OnComplete (bool);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End of MGUI namespace

#endif		//	INC_MGUI_FORMPASSWORD_H
