/**
 * \file mi32/containeritemsync.h
 * \brief Definition for Container Item Synchronizer
 *
 * \if NODOC
 * $Id: containeritemsync.h_v 1.6 2011/10/13 12:53:54 mju Exp $
 *
 * $Log: containeritemsync.h_v $
 * Revision 1.6  2011/10/13 12:53:54  mju
 * Remove redundant hdr.
 *
 * Revision 1.5  2008/09/02 17:54:32  mju
 * Fix doxygen warning.
 *
 * Revision 1.4  2007/10/18 16:23:02  scowan
 * Added doc use code example.
 *
 * Revision 1.3  2007/08/22 21:48:22  scowan
 * Uses interlocked int32 class, object is now inlined.
 *
 * Revision 1.1  2007/08/17 21:03:00  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_CONTAINERITEMSYNC_H
#define	INC_MI32_CONTAINERITEMSYNC_H

#ifndef	INC_MI32_CRITSEC_H
	#include <mi32/critsec.h>
#endif

#ifndef	INC_MI32_INTERLOCKEDINT32_H
	#include <mi32/interlockedint32.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace MITHREAD {

//! This class is a solution to the issue of synchronizing container contents without using a synchronization object
//! per container item.
//! Example below:
//! @code
//!	//! Thread local storage container class for thread error state
//! 	class CONTAINER {
//! 		public:
//!			class ITEM {
//!				...
//!				private:
//!					MITHREAD::CONTAINERITEMSYNC::OBJECT m_ItemLock;
//!					...
//!				};
//! 			...
//!
//! 			ITEM& GetItem (
//!				INT32 ItemNum
//! 				) {
//!				...
//!				// Wait until other thread has released its lock on 'ItemNum'
//!				m_ContainerSync.AcquireSyncObject(ItemNum, m_ItemArray[ItemNum].m_ItemLock);
//!				...
//! 				return (m_ItemArray[ItemNum]);		// Exclusive ownership granted to 'ItemNum' for the life of the calling code's ITEM instance. 
//!				}
//!		
//! 		private:
//!			std::vector<ITEM> m_ItemArray;
//!			MITHREAD::CONTAINERITEMSYNC m_ContainerSync;
//!			...
//! 		};
//!
//! @endcode
class CLASSLIBEXPORT CONTAINERITEMSYNC {
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	public:
		//! This class holds the lock on the synchronization object until the dtor or the Release() method is called.
		//! Place an instance of this class in another class that controls access to the container item's contents to
		//! use scoping rules to cause the lock to be released when the container's item access is complete.
		class OBJECT {
			public:
				//! Default ctor
				OBJECT (
					) : m_Counter(0) { }
				
				//! Default dtor.
				//! Calls Release() to give up its lock.
				~OBJECT (
					) { Release(); }
			
				//! Release the lock on the synchronization object.  The container item is no longer synchronized 
				//! after a call to this method.
				void Release (
					) {
					if (m_Counter != 0) {
						(*m_Counter)--;
						m_CacheLock.Release();
						m_Counter = 0;
						}
					return;
					}
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! The members below should not be hidden due to the necessity of maximum speed.
				//! The call 'new PRIV' is too expensive for where this class needs to be used.
				CRITICALSECTION::LOCK m_CacheLock;
				INTERLOCKEDINT32* m_Counter;

				OBJECT (const OBJECT& rhs);
				OBJECT& operator=(const OBJECT& rhs);

				friend class CONTAINERITEMSYNC::PRIV;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Default ctor
		CONTAINERITEMSYNC ();

		//! Default dtor
		~CONTAINERITEMSYNC ();

		//! Acquire a locked synchronization object for the 'ItemID' specified.
		//! If an 'ItemID' is in the cache, this method will block until the other thread's lock 
		//! on that 'ItemID' is released and the lock is granted for the calling thread.
		//! This method will call SyncObject.Release() before acquiring a new lock.
		void AcquireSyncObject (
			PTRUINT ItemID,			//!< This value must be unique in regards to the container.
			OBJECT& SyncObject		//!< Either the dtor or the Release() method will release this thread's lock on 'ItemID'.
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PRIV* m_pPriv;

		CONTAINERITEMSYNC (const CONTAINERITEMSYNC& rhs);
		CONTAINERITEMSYNC& operator=(const CONTAINERITEMSYNC& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of MITHREAD namespace

#undef	CLASSLIBEXPORT
#endif	//!< INC_MI32_CONTAINERITEMSYNC_H

