/**
 * \file mi32/filetype.h
 * \brief FILETYPE class definition
 *
 * \if NODOC
 * $Id: filetype.h_v 1.7 2010/08/20 17:55:36 mju Exp $
 *
 * $Log: filetype.h_v $
 * Revision 1.7  2010/08/20 17:55:36  mju
 * Don't' use const for file_raw in delegate_format or isFormat.
 *
 * Revision 1.6  2009/12/18 18:39:15  bcolombe
 * Added SetDelegateIsFormat and SetPropertiesDelegate
 *
 * Revision 1.5  2009/12/18 17:41:47  bcolombe
 * Remove v_GetProperties, use constructor with delegate
 *
 * Revision 1.4  2009/12/17 23:22:50  bcolombe
 * Added GetProperties and v_GetProperties
 *
 * Revision 1.3  2009/11/04 22:38:38  mju
 * Add classlibexport.
 *
 * Revision 1.2  2009/11/04 18:56:44  mju
 * Move FILETYPESET to separate header.
 *
 * Revision 1.1  2009/11/04 18:23:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FILETYPE_H
#define	INC_MI32_FILETYPE_H

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class MISTRINGLIST;
class FILE_RAW;
class FILEPATH;
#endif

//=====================================================================================================================

//! File type definition, including extensions, description, icon and optional delegate for determining if is correct format.
//! Instances are reference counted so may be copied efficiently.
class CLASSLIBEXPORT FILETYPE {
	public:
		//! Delegate to determine if file is of defined type.
		typedef fastdelegate::FastDelegate<bool(FILE_RAW&)> DELEGATE_FORMAT;

		//! Delegate to get properties
		typedef fastdelegate::FastDelegate<void(MISTRING&, const FILEPATH&)> PROPERTIES_DELEGATE;

		//! Default constructor.
		FILETYPE (
			);

		//! Copy constructor.
		FILETYPE (
			const FILETYPE& rhs
			);

		//! Construct with char* extension(s), description, icon and no delegate.
		explicit FILETYPE (
			const char *extns,								//!< One or more extensions, separated by space, do not include wildcard (*) or period (.), blank if no extension.
			ICONID iconid = ICONID_DESKTOP_FILE,			//!< Icon ID
			const TEXTID description = TEXTID__None			//!< Description TEXTID, if not provided will determine from extensions
			);

		//! Construct with char* extension(s), description, icon and format delegate.
		FILETYPE (
			const char *extns,								//!< One or more extensions, separated by space, do not include wildcard (*) or period (.), blank if no extension.
			ICONID iconid,									//!< Icon ID
			const TEXTID description,						//!< Description TEXTID, if none provided will determine from extensions
			DELEGATE_FORMAT delegate						//!< Delegate to test if file is actually of specified type
			);

		//! Construct with char* extension(s), description, icon and properties delegate.
		FILETYPE (
			const char *extns,								//!< One or more extensions, separated by space, do not include wildcard (*) or period (.), blank if no extension.
			ICONID iconid,									//!< Icon ID
			const TEXTID description,						//!< Description TEXTID, if none provided will determine from extensions
			PROPERTIES_DELEGATE delegate					//!< Delegate to get properties
			);

		//! Construct with char* extension(s), description, icon, format delegate and properties delegate.
		FILETYPE (
			const char *extns,								//!< One or more extensions, separated by space, do not include wildcard (*) or period (.), blank if no extension.
			ICONID iconid,									//!< Icon ID
			const TEXTID description,						//!< Description TEXTID, if none provided will determine from extensions
			DELEGATE_FORMAT formatdelegate,					//!< Delegate to test if file is actually of specified type
			PROPERTIES_DELEGATE propertiesdelegate			//!< Delegate to get properties
			);

		//! Construct with MISTRING extension.
		explicit FILETYPE (
			const MISTRING& extn,							//!< Single extension, do not include wildcard (*) or period (.).
			ICONID iconid = ICONID_DESKTOP_FILE			//!< Icon ID
			);

		//! Destructor.
		virtual ~FILETYPE (
			);

		//! Assignment.
		FILETYPE& operator= (
			const FILETYPE& rhs
			);

		//! Determine if can check if file is correct format.
		bool CanCheckIsFormat (
			) const;

		//! Get file type description.
		const MISTRING& GetDescription (
			) const;

		//! Get extension list.
		const MISTRINGLIST& GetExtnList (
			) const;

		//! Get icon ID.
		ICONID GetIconID (
			) const;

		//! Get properties
		void GetProperties (
			MISTRING& properties,
			const FILEPATH& filepath
			) const;

		//! Determine if file is in correct format for type.
		//! Will return true if no delegate.
		bool IsFormat (
			FILE_RAW& file
			) const;

		//! Register this type in common set.
		void RegisterInCommonSet (
			) const;

		//! Set delegate to test if file is actually of specified type
		void SetDelegateIsFormat (
			DELEGATE_FORMAT delegate
			);

		//! Set delegate to get properties
		void SetPropertiesDelegate (
			PROPERTIES_DELEGATE delegate
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

//=====================================================================================================================

#undef	CLASSLIBEXPORT
#endif	// INC_MI32_FILETYPE_H
