/**
 * \file imgtilit.h <mi32/imgtilit.h>
 * \brief Definitions for IMAGE_TILE_ITERATOR class.
 *
 * \if NODOC
 * $Id: imgtilit.h_v 1.11 2009/08/13 19:34:41 mju Exp $
 *
 * $Log: imgtilit.h_v $
 * Revision 1.11  2009/08/13 19:34:41  mju
 * Support having different size for first tile.
 *
 * Revision 1.10  2007/01/25 14:35:55  mju
 * Add getMaxColumns.
 *
 * Revision 1.9  2007/01/25 14:17:23  mju
 * Remove rvcrastinfo reference.
 *
 * Revision 1.8  2006/03/15 16:43:01  drb
 * Added documentation to require the object be opened first
 *
 * Revision 1.7  2006/03/15 16:31:13  drb
 * Added constructor that takes RVC::RASTER
 *
 * Revision 1.1  2000/05/03 14:59:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_IMGTILIT_H
#define	INC_MI32_IMGTILIT_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class RASTER;
	}
#endif

//! \class IMAGE_TILE_ITERATOR mi32/imgtilit.h
//! \brief Allow iteration through an image in tile order.
//!
//! In many cases, all cells in an image are to be processed, but
//! the order is not important to the algorithm.  In this case, it
//! is most efficient to read or write the image in tile order,
//! processing all cells in one tile before moving to the next tile.
//! This class simplifies and encapsulates the details of iterating
//! through the tiles and spans within each tile.
//!
//! The following code fragment shows how this class is used for RVC rasters
//! to compute the minimum and maximum cell values in the raster.
//! Error checking and null cell handling issues are ignored for simplicity.
//!
//!	@code
//!		RVC::RASTER ObjRaster;
//!		ObjRaster.Open(ObjItemRaster);
//!		IMAGE_TILE_ITERATOR it(ObjRaster);
//!		DOUBLE_ARRAY<double> rowbuf;
//!		rowbuf.Resize(it.GetMaxColumns());
//!		double minval = DBL_MAX, maxval = DBL_MIN;
//!		for (; (!it.IsDone()); ++it) {
//!			ObjRaster.Read(it.GetRow(),it.GetColumn(),rowbuf,it.GetNumColumns());
//!			for (INT32 col = 0; (col < it.GetNumColumns()); ++col) {
//!				if (rowbuf[col] < minval) minval = rowbuf[col];
//!				if (rowbuf[col] > maxval) maxval = rowbuf[col];
//!				}
//!			}
//!	@endcode

class IMAGE_TILE_ITERATOR {
	public:

		//! Construct iterator using raster object
		IMAGE_TILE_ITERATOR (
			const RVC::RASTER& ObjRaster				//!< RVC::RASTER object that is already opened
			);

		//! Construct iterator for given image and tile size
		IMAGE_TILE_ITERATOR (
			INT32 numcols,									//!< Number of columns in image
			INT32 numrows,									//!< Number of rows in image
			INT32 tilecols,								//!< Number of columns in a single tile
			INT32 tilerows									//!< Number of rows in a single tile
			);

		//! Construct iterator for given image and tile size with different size for first tile row/column
		IMAGE_TILE_ITERATOR (
			INT32 numcols,									//!< Number of columns in image
			INT32 numrows,									//!< Number of rows in image
			INT32 tilecols,								//!< Number of columns in a single tile
			INT32 tilerows,								//!< Number of rows in a single tile
			INT32 FirstTileColumns,						//!< Number of columns in first tile
			INT32 FirstTileRows							//!< Number of rows in first tile
			);

		//! Increment to next span.
		IMAGE_TILE_ITERATOR& operator++ (
			);

		//! Get starting column for current span.
		INT32 GetColumn (
			) const {
			return (m_col0);
			}

		//! Get row within image for current span.
		INT32 GetLine (
			) const {
			return (m_row);
			}

		//! Get maximum number of columns.
		INT32 GetMaxColumns (
			) const { return (MAX(m_tilecols,m_tilecolsfirst)); }

		//! Get number of columns in current span.
		INT32 GetNumColumns (
			) const {
			return (m_col1 - m_col0);
			}

		//! Get total number of spans.
		INT32 GetNumSpans (
			) const {
			return (m_numspans);
			}

		//! Get row within image for current span.
		INT32 GetRow (
			) const {
			return (m_row);
			}

		//! Get current span number.
		INT32 GetSpan (
			) const {
			return (m_curspan);
			}

		//! Determine if at end of image.
		bool IsDone (
			) const {
			return (m_Done);
			}

		//! Reset to beginning.
		void Reset (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_numcols;
		INT32 m_numrows;
		INT32 m_tilecols;
		INT32 m_tilerows;
		INT32 m_tilecolsfirst;
		INT32 m_tilerowsfirst;
		INT32 m_numspans;
		INT32 m_curspan;
		INT32 m_row0;
		INT32 m_row1;
		INT32 m_col0;
		INT32 m_col1;
		INT32 m_row;
		bool m_Done;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_IMGTILIT_H
