/**
 * \file mi32/mg2imagefilewriter.h
 * \brief Class MGD::IMAGEFILEWRITER - Support for writing to image file formats
 *
 * \if NODOC
 *
 * $Id: mg2imagefilewriter.h_v 1.6 2012/08/21 21:37:27 mju Exp $
 *
 * $Log: mg2imagefilewriter.h_v $
 * Revision 1.6  2012/08/21 21:37:27  mju
 * Add formats for kmz.
 *
 * Revision 1.5  2006/12/05 18:27:15  mju
 * Change objitem to non-const so can return location written to.
 *
 * Revision 1.4  2006/04/11 21:55:33  mju
 * Add parm to useXasAlpha.
 *
 * Revision 1.3  2006/01/05 18:48:45  mju
 * Add write using objitem.
 *
 * Revision 1.1  2005/12/08 22:39:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MG2IMAGEFILEWRITER_H
#define	INC_MI32_MG2IMAGEFILEWRITER_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class FILEPATH;
class TRANS2D_MAPGEN;
namespace SPATREF {
	class COORDREFSYS;
	}
namespace MGD {
	class DEVICE;
	}
namespace RVC {
	class OBJITEM;
	}
#endif

namespace MGD {

class IMAGEFILEWRITER {
	public:

		//! Container for settings specifying format and compression.
		class SETTINGS {
			public:

				class FORM;

				//! Image file format.
				enum FORMAT {
					FORMAT_RVC =		0x00000100,
					FORMAT_GeoJP2 =	0x00000200,
					FORMAT_JP2 =		0x00000400,
					FORMAT_GeoTIFF =	0x00000800,
					FORMAT_TIFF =		0x00001000,
					FORMAT_JPEG =		0x00002000,
					FORMAT_PNG =		0x00004000,
					FORMAT_KMZBASE =	0x10000000,
					FORMAT_KMZ_JPEG =	FORMAT_KMZBASE | FORMAT_JPEG,
					FORMAT_KMZ_PNG =	FORMAT_KMZBASE | FORMAT_PNG,
					FORMAT_MASK =		0xFFFFFF00		// Keep low bits free so can combine with compression type.
					};

				//! Compression type enumeration.
				enum COMPRESSTYPE {
					COMPRESSTYPE_None =						0,		// RVC, TIFF
		//			COMPRESSTYPE_RLE =						1,		// RVC
					COMPRESSTYPE_DPCM =						2,		// RVC
					COMPRESSTYPE_Huffman =					3,		// RVC
					COMPRESSTYPE_LZW =						4,		// TIFF
					COMPRESSTYPE_PackBits =					5,		// TIFF
					COMPRESSTYPE_JPEG_BestQuality =		6,		// RVC, JPEG
					COMPRESSTYPE_JPEG_UserQuality =		7,		// RVC, JPEG
					COMPRESSTYPE_JPEG2000_Lossless =		8,		// RVC, JP2
					COMPRESSTYPE_JPEG2000_BestQuality =	9,		// RVC, JP2
					COMPRESSTYPE_JPEG2000_UserRatio =	10,	// RVC, JP2
					};

				//! Default constructor.
				//! Initializes to RVC uncompressed.
				SETTINGS ();

				//! Get compression ratio.
				//! @return compression ratio, 0 for lossless, 1 for best quality, > 1 for specified ratio
				//! Applies to JPEG2000 compression types.
				UINT16 GetCompressRatio () const
					{ return (m_CompressRatio); }

				//! Get compression quality.
				//! @return compression quality, 100 for best.
				//! Applies to JPEG compression types.
				UINT16 GetCompressQuality () const
					{ return (m_CompressQuality); }

				//! Get compression types.
				COMPRESSTYPE GetCompressType () const
					{ return (m_CompressType); }

				//! Get file extension based on current format.
				//! @return Extension with no "." (eg "tif").
				const char* GetFileExtension () const;

				//! Get format.
				FORMAT GetFormat () const
					{ return (m_Format); }

				//! Read settings from INI file.
				//! @return 'true' if the entry exists.
				bool IniRead (
					void *IniHandle,						//!< Handle to INI file to read from, 0 for default file
					const char *group,					//!< INI group to read from
					const char *field						//!< INI field to read from
					);

				//! Write settings to INI file.
				void IniWrite (
					void *IniHandle,						//!< Handle to INI file to write to, 0 for default file
					const char *group,					//!< INI group to write to
					const char *field						//!< INI field to write to
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				FORMAT m_Format;
				COMPRESSTYPE m_CompressType;
				UINT16 m_CompressQuality;
				UINT16 m_CompressRatio;

				friend class SETTINGS::FORM;
				#endif	// GENERATING_DOXYGEN_OUTPUT
			};

		IMAGEFILEWRITER (
			const SETTINGS& settings
			);

		~IMAGEFILEWRITER ();

		ERRVALUE Write (
			const FILEPATH& filepath,
			MGD::DEVICE& DeviceSrc,
			const TRANS2D_MAPGEN *pTransToCRS = 0,
			bool UseXasAlpha = false
			);

		ERRVALUE Write (
			RVC::OBJITEM& objitem,					//!< Object location passed/returned
			MGD::DEVICE& DeviceSrc,
			const TRANS2D_MAPGEN *pTransToCRS = 0,
			bool UseXasAlpha = false
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SETTINGS m_Settings;
		#endif
	};


}	// End namespace MGD

#endif	// INC_MI32_MG2IMAGEFILEWRITER_H
