/**
 * \file mi32/samclass.h
 * \brief Definition for SAMCLASSIFIER (Spectral Angle Mapper Classifier)
 *
 * \if NODOC
 * $Id: samclass.h_v 1.13 2011/11/07 14:06:36 mju Exp $
 *
 * $Log: samclass.h_v $
 * Revision 1.13  2011/11/07 14:06:36  mju
 * Remove redundant hdrs.
 *
 * Revision 1.12  2007/03/02 17:02:11  honerkamp
 * convert to use RVC classes
 *
 * Revision 1.11  2006/07/26 23:01:28  scowan
 * rvc interface class clean.
 *
 * Revision 1.10  2005/12/12 14:38:22  mju
 * Eliminate inline so no deprecation warning.
 *
 * Revision 1.9  2005/07/18 13:21:00  mju
 * Use mi math hdr instead of ieefp hdr.
 *
 * Revision 1.8  2005/03/31 16:57:20  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.7  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.6  2000/10/30 20:26:44  msmith
 * Genitor documentation.
 *
 * Revision 1.5  2000/10/16 13:50:10  mju
 * Inc rvcrast.h.
 *
 * Revision 1.4  1999/09/03 16:10:51  mju
 * Redesign to use classes.
 *
 * Revision 1.3  1999/08/19  20:19:34  dfrolov
 * add flags
 *
 * Revision 1.2  1999/08/10  21:03:48  dfrolov
 * add MfClassifySAMClassisValid() function
 *
 * Revision 1.1  1999/08/10  20:20:07  dfrolov
 * Initial revision
 *
 * \endif
************************************************************************/

#ifndef INC_MI32_SAMCLASS_H
#define INC_MI32_SAMCLASS_H

#ifndef	INC_RVC_HYPSPECT_H
	#include <rvc/hypspect.h>
#endif

#ifndef INC_RVC_HYPERSENSOR_H
	#include <rvc/hypersensor.h>
#endif

#ifndef INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif


//! Spectral Angle Mapper classifier.
class SAMCLASSIFIER {
	public:

		//! Constructor.
		SAMCLASSIFIER (
			RVC::OBJITEM &objitem		//!< Hyperspectral objitem
			);

		//! Destructor.
		~SAMCLASSIFIER (
			);

		//! Add sample point to classifier.
		int AddSample (
			INT32 lin,								//!< Sample point - lin
			INT32 col								//!< Sample point - col
			);

		//! Setup classification.
		int ClassifyBegin (
			);

		//! Do classification on specified line.
		int ClassifyLine (
			UINT32 lin,								//!< Line to classify
			UINT32 startcol,						//!< Starting column of line
			UINT32 numcols,						//!< Number of columns
			UINT8 *outmask							//!< Which columns to use
			);

		//! Close hyperspectral object.
		void Close (
			);

		//! Delete all sample points.
		void DeleteAllSamples (
			);

		//! Delete sample point.
		void DeleteSample (
			INT32 lin,								//!< Sample point - lin
			INT32 col								//!< Sample point - col
			);

		//! Open hyperspectral object.
		ERRVALUE Open (
			);

		//! Set average sample value.
		void SetAverageSamples (
			bool AverageSamples = true			//!< Average samples
			) { m_AverageSamples = AverageSamples; }

		//! Set ignore absorption bands.
		void SetIgnoreAbsorptionBands (
			bool IgnoreAbsorptionBands = true//!< Ignore absorption bands
			) { m_IgnoreAbsorptionBands = IgnoreAbsorptionBands; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class SAMPLE {
			public:
				//! Constructor
				explicit SAMPLE (
					SAMCLASSIFIER& sam,				//!< Classifier
					int lin = 0,						//!< Lines
					int col = 0							//!< Columns
					);

				//! Destructor
				~SAMPLE (
					);

				//! Add to the accumulators.
				void AccumAdd (
					INT32 band,							//!< Band to multiply by
					INT32 col,							//!< Column to add to / from
					INT16* linebuf						//!< Values to add
					) {
					m_SumTR[col] += linebuf[col] * m_values[band];
					m_SumTT[col] += linebuf[col] * linebuf[col];
					}

				//! Initializes the accumulators.
				int AccumInit (
					);

				//! Allocate the values member.
				int AllocValues (
					);

				//! Compute the angle of a given column.
				double ComputeAngle (
					INT32 col							//!< Column to calculate
					) const { return (acos(m_SumTR[col] / (sqrt(m_SumTT[col]) * m_magnitude))); }

				//! Compute the magnitude.
				void ComputeMagnitude (
					);

				//! Read the value of a given band.
				int ReadValue (
					int band								//!< Band to read from
					);
					
			private:
				SAMCLASSIFIER& m_sam;				//!< Classifier
				INT32 m_lin;							//!< Lines
				INT32 m_col;							//!< Columns
				double m_magnitude;					//!< Magnitude of "m_values" vector
				double *m_values;						//!< Values
				double *m_SumTR;						//!< Accumulator
				double *m_SumTT;						//!< Accumulator

				friend class SAMCLASSIFIER;
			};

		RVC::OBJITEM m_HyperObjItem;
		RVC::HYPERSPECTRAL m_HyperSpecObj;
		bool m_IgnoreAbsorptionBands;			//!< Ignore absorption bands flag
		bool m_AverageSamples;					//!< Average samples flag
		bool m_ProcessLossy;						//!< Lossy processing flag
		double m_Tolerance;						//!< Tolerance
		double m_AvgTolerance;					//!< Average tolerance
		BITSET m_ProcBand;						//!< Bitset indicating if band should be processed
		DOUBLE_ARRAY<RVC::HYPERSENSOR::BANDINFO> m_BandInfo;	//!< Sensor band info
		int m_NumSamples;							//!< Number of samples
		SAMPLE **m_Samples;						//!< List of samples
		SAMPLE m_AvgSample;						//!< Average sample
		INT16 *m_linebuf;							//!< Line buffer

		friend class SAMCLASSIFIER::SAMPLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#endif
