/**
 * \file mi32/trans2daffine.h
 * \brief TRANS2D_AFFINE definitions
 *
 * \if NODOC
 * $Id: trans2daffine.h_v 1.7 2011/09/23 15:34:51 mju Exp $
 *
 * $Log: trans2daffine.h_v $
 * Revision 1.7  2011/09/23 15:34:51  mju
 * Fwd declare filepath.
 *
 * Revision 1.6  2011/09/23 13:40:13  mju
 * Add combineForwardReverse.
 *
 * Revision 1.5  2011/09/22 21:44:10  mju
 * Don't inline rectangle conversions.
 *
 * Revision 1.4  2011/09/22 20:58:20  mju
 * Add convert using general array.
 *
 * Revision 1.3  2011/09/21 21:04:06  mju
 * Need rect hdr after all.
 *
 * Revision 1.2  2011/09/21 19:27:41  mju
 * Remove ctrlpt and rect hdrs.
 *
 * Revision 1.1  2011/09/21 18:09:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TRANS2DAFFINE_H
#define	INC_MI32_TRANS2DAFFINE_H

#ifndef	INC_MI32_TRANS2D_H
#include <mi32/trans2d.h>
#endif

#ifndef	INC__MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MEMORY_H
#include <memory.h>				// For memset/memcpy
#define	INC_MEMORY_H
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
#ifdef GEOMDLL
	#define LIBEXPORT MI_DLLEXPORT
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

// Forward declarations
class TRANS2D_MAPGEN;
class FILEPATH;

#endif // GENERATING_DOXYGEN_OUTPUT


//-------------------------------------------------------------------------------------------------------------------
//! Simple affine X/Y coordinate transformation.
//!
//! All conversion methods are inline for maximum speed.  For single point
//! conversion, no function call will occur (assuming the compiler actually
//! inlines the method).  This class replaces the trans2d...() C functions.
class CLASSLIBEXPORT TRANS2D_AFFINE {

	public:

		enum AXIS {
			AXIS_X = 0,
			AXIS_Y = 1
			};

		//! Default constructor, create identity transformation.
		TRANS2D_AFFINE (
			) { trans2dinit(m_fwd,m_inv); }

		//! Copy constructor.
		TRANS2D_AFFINE (
			const TRANS2D_AFFINE& rhs
			) {
			memcpy(m_fwd,rhs.m_fwd,sizeof(MAT3X3));
			memcpy(m_inv,rhs.m_inv,sizeof(MAT3X3));
			}

		//! Construct from forward or inverse matrix
		TRANS2D_AFFINE (
			const MAT3X3 matrix,					//!< Transformation matrix
			bool IsForward = true				//!< True if forward transformation, false if inverse
			) {
			if (IsForward) {
				memcpy(m_fwd,matrix,sizeof(MAT3X3));
				trans2dfindinverse(m_fwd,m_inv);
				} 
			else {
				memcpy(m_inv,matrix,sizeof(MAT3X3));
				trans2dfindinverse(m_inv,m_fwd);
				}
			}

		//! Construct from affine matrices.
		TRANS2D_AFFINE (
			const MAT3X3 fwd,						//!< Forward transformation matrix
			const MAT3X3 inv						//!< Inverse transformation matrix
			) {
			memcpy(m_fwd,fwd,sizeof(MAT3X3));
			memcpy(m_inv,inv,sizeof(MAT3X3));
			}

		//! Construct from combination of two affine transformations.
		TRANS2D_AFFINE (
			const TRANS2D_AFFINE& trans1,		//!< First transformation
			const TRANS2D_AFFINE& trans2		//!< Second transformation
			) { trans2dmatrix2(m_fwd,m_inv,trans1.m_fwd,trans1.m_inv,trans2.m_fwd,trans2.m_inv); }

		//! Construct from CTRLPOINT array.
		TRANS2D_AFFINE (
			const CTRLPOINT *cp,					//!< Control point array
			int numpoints,							//!< Number of control points
			bool invert = false					//!< Invert coordinate system (for images)
			) { FindBestTrans2DA(numpoints,cp,m_fwd,m_inv,(invert)?TRANS2DMODEL_OrientInvert:0); }

		//! Construct from CTRLPOINT3 array.
		TRANS2D_AFFINE (
			const CTRLPOINT3 *cp,				//!< Control point array
			int numpoints,							//!< Number of control points
			bool invert = false					//!< Invert coordinate system (for images)
			) { FindBestTrans2D3A(numpoints,cp,m_fwd,m_inv,(invert)?TRANS2DMODEL_OrientInvert:0); }

		//! Destructor.
		~TRANS2D_AFFINE (
			) {}

		//! Assignment from TRANS2D_AFFINE.
		TRANS2D_AFFINE& operator= (
			const TRANS2D_AFFINE& rhs
			) {
			if (this != &rhs) {
				memcpy(m_fwd,rhs.m_fwd,sizeof(MAT3X3));
				memcpy(m_inv,rhs.m_inv,sizeof(MAT3X3));
				}
			return (*this);
			}

		//! Apply coordinate offset.
		void ApplyOffset (
			double xoffset,						//!< X offset
			double yoffset							//!< Y offset
			) { trans2dshift(m_fwd,m_inv,xoffset,yoffset); }

		//! Apply coordinate offset.
		void ApplyOffset (
			const DPOINT2D& offset				//!< Offset
			) { trans2dshift(m_fwd,m_inv,offset.x,offset.y); }

		//! Apply coordinate offset relative to reverse direction.
		void ApplyOffsetReverse (
			double xoffset,						//!< X offset
			double yoffset							//!< Y offset
			) { trans2dshift(m_inv,m_fwd,xoffset,yoffset); }

		//! Apply coordinate offset relative to reverse direction.
		void ApplyOffsetReverse (
			const DPOINT2D& offset				//!< Offset
			) { trans2dshift(m_inv,m_fwd,offset.x,offset.y); }

		//! Apply rotation angle.
		void ApplyRotation (
			double angle							//!< Rotation angle in radians
			) { trans2drot(m_fwd,m_inv,angle); }

		//! Apply scale equally to X/Y axes.
		void ApplyScale (
			double scale							//!< Scale
			) { trans2dscale(m_fwd,m_inv,scale,scale); }

		//! Apply scale to X/Y axes.
		void ApplyScale (
			double xscale,							//!< X scale
			double yscale							//!< Y scale
			) { trans2dscale(m_fwd,m_inv,xscale,yscale); }

		//! Apply scale to X/Y axes.
		void ApplyScale (
			const DPOINT2D &scale				//!< X/Y scale
			) { trans2dscale(m_fwd,m_inv,scale.x,scale.y); }

		//! Apply scale to X/Y axes, relative to reverse direction.
		void ApplyScaleReverse (
			double xscale,							//!< X scale
			double yscale							//!< Y scale
			) { trans2dscale(m_inv,m_fwd,xscale,yscale); }

		//! Apply scale to X/Y axes, relative to reverse direction.
		void ApplyScaleReverse (
			const DPOINT2D &scale				//!< X/Y scale
			) { trans2dscale(m_inv,m_fwd,scale.x,scale.y); }

		//! Apply shear relative to specified axis.
		void ApplyShear (
			double angle,							//!< Shear angle in radians
			AXIS axis								//!< Axis to shear (AXIS_X or AXIS_Y)
			) { trans2dshear(m_fwd,m_inv,angle,static_cast<int>(axis)); }

		//! Combine two affine transformations.
		void Combine (
			const TRANS2D_AFFINE& trans1		//!< First transformation
			) {
			trans2dmatrix(m_fwd,trans1.m_fwd);
			trans2dmatrix(m_inv,trans1.m_inv);
			}

		//! Combine two affine transformations, both in forward direction.
		void Combine (
			const TRANS2D_AFFINE& trans1,		//!< First transformation
			const TRANS2D_AFFINE& trans2		//!< Second transformation
			) { trans2dmatrix2(m_fwd,m_inv,trans1.m_fwd,trans1.m_inv,trans2.m_fwd,trans2.m_inv); }

		//! Combine two affine transformations, first forward, second reversed.
		void CombineForwardReverse (
			const TRANS2D_AFFINE& trans1,		//!< First transformation
			const TRANS2D_AFFINE& trans2		//!< Second transformation
			) { trans2dmatrix2(m_fwd,m_inv,trans1.m_fwd,trans1.m_inv,trans2.m_inv,trans2.m_fwd); }

		//! Perform forward transformation on X/Y point.
		//! @return Converted point.
		DPOINT2D ConvertForward (
			double x,								//!< X coordinate
			double y									//!< Y coordinate
			) const { return (DPOINT2D(trans2dx(x,y,m_fwd),trans2dy(x,y,m_fwd))); }

		//! Perform forward transformation on X/Y point.
		void ConvertForward (
			double x,								//!< X coordinate
			double y,								//!< Y coordinate
			double& ox,								//!< X coordinate returned
			double& oy								//!< Y coordinate returned
			) const { ox = trans2dx(x,y,m_fwd); oy = trans2dy(x,y,m_fwd); }

		//! Perform forward transformation on DPOINT2D.
		//! @return Converted point.
		DPOINT2D ConvertForward (
			const DPOINT2D& ipoint				//!< Input point
			) const { return (DPOINT2D(trans2dx(ipoint.x,ipoint.y,m_fwd),trans2dy(ipoint.x,ipoint.y,m_fwd))); }

		//! Perform forward transformation on DPOINT3D.
		//! @return Converted point.
		DPOINT3D ConvertForward (
			const DPOINT3D& ipoint				//!< Input point
			) const { return (DPOINT3D(trans2dx(ipoint.x,ipoint.y,m_fwd),trans2dy(ipoint.x,ipoint.y,m_fwd),ipoint.z)); }

		//! Perform forward transformation on DPOINT2D array.
		void ConvertForward (
			const DPOINT2D *ipoints,			//!< Input point array
			DPOINT2D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform forward transformation on DPOINT3D array.
		void ConvertForward (
			const DPOINT3D *ipoints,			//!< Input point array
			DPOINT3D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform forward transformation on general array.
		void ConvertForward (
			const double *ipoints,				//!< Input point array
			double *opoints,						//!< Output point array
			INT32 NumPoints,						//!< Number of points to convert
			int NumDim								//!< Number of dimensions
			) const;

		//! Perform forward transformation on DRECT2D.
		//! @return Converted rectangle.
		DRECT2D ConvertForward (
			const DRECT2D& irect					//!< Input rectangle
			) const {
			DRECT2D orect;
			ConvertForward(irect,orect);
			return (orect);
			}

		//! Perform forward transformation on DRECT2D.
		void ConvertForward (
			const DRECT2D& irect,				//!< Input rectangle
			DRECT2D& orect							//!< Output rectangle
			) const;

		//! Perform inverse transformation on X/Y point.
		//! @return Converted point.
		DPOINT2D ConvertInverse (
			double x,								//!< X coordinate
			double y									//!< Y coordinate
			) const { return (DPOINT2D(trans2dx(x,y,m_inv),trans2dy(x,y,m_inv))); }

		//! Perform forward transformation on X/Y point.
		void ConvertInverse (
			double x,								//!< X coordinate
			double y,								//!< Y coordinate
			double& ox,								//!< X coordinate returned
			double& oy								//!< Y coordinate returned
			) const { ox = trans2dx(x,y,m_inv); oy = trans2dy(x,y,m_inv); }

		//! Perform inverse transformation on DPOINT2D.
		//! @return Converted point.
		DPOINT2D ConvertInverse (
			const DPOINT2D& ipoint				//!< Input point
			) const { return (DPOINT2D(trans2dx(ipoint.x,ipoint.y,m_inv),trans2dy(ipoint.x,ipoint.y,m_inv))); }

		//! Perform inverse transformation on DPOINT3D.
		//! @return Converted point.
		DPOINT3D ConvertInverse (
			const DPOINT3D& ipoint				//!< Input point
			) const { return (DPOINT3D(trans2dx(ipoint.x,ipoint.y,m_inv),trans2dy(ipoint.x,ipoint.y,m_inv),ipoint.z)); }

		//! Perform inverse transformation on DPOINT2D array.
		void ConvertInverse (
			const DPOINT2D *ipoints,			//!< Input point array
			DPOINT2D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform inverse transformation on DPOINT3D array.
		void ConvertInverse (
			const DPOINT3D *ipoints,			//!< Input point array
			DPOINT3D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform reverse transformation on general array.
		void ConvertInverse (
			const double *ipoints,				//!< Input point array
			double *opoints,						//!< Output point array
			INT32 NumPoints,						//!< Number of points to convert
			int NumDim								//!< Number of dimensions
			) const;

		//! Perform inverse transformation on DRECT2D.
		//! @return Converted rectangle.
		DRECT2D ConvertInverse (
			const DRECT2D& irect					//!< Input rectangle
			) const {
			DRECT2D orect;
			ConvertInverse(irect,orect);
			return (orect);
			}

		//! Perform inverse transformation on DRECT2D.
		void ConvertInverse (
			const DRECT2D& irect,				//!< Input rectangle
			DRECT2D& orect							//!< Output rectangle
			) const;

		//! Determine scale/rotation/shear for forward transformation
		void DisectForward (
			double *xscale,						//!< X scale returned (0 if don't need)
			double *yscale,						//!< Y scale returned (0 if don't need)
			double *rotangle = 0,				//!< Rotation angle in radians returned (0 if don't need)
			double *shearangle = 0				//!< Shear angle in radians returned (0 if don't need)
			) const { DisectTrans2D(m_fwd,xscale,yscale,rotangle,shearangle); }
			
		//! Determine scale/rotation/shear for inverse transformation
		void DisectInverse (
			double *xscale,						//!< X scale returned (0 if don't need)
			double *yscale,						//!< Y scale returned (0 if don't need)
			double *rotangle = 0,				//!< Rotation angle in radians returned (0 if don't need)
			double *shearangle = 0				//!< Shear angle in radians returned (0 if don't need)
			) const { DisectTrans2D(m_inv,xscale,yscale,rotangle,shearangle); }

		//! Retrieve copy of affine matrices		
		void GetMatrices (
			MAT3X3 fwd,								//!< Forward matrix returned
			MAT3X3 inv								//!< Inverse matrix returned
			) const { memcpy(fwd,m_fwd,sizeof(MAT3X3)); memcpy(inv,m_inv,sizeof(MAT3X3)); }

		//! Get values for forward transformation.
		void GetValuesForward (
			double& xscale,						//!< X scale value returned
			double& xrot,							//!< X rotation value returned
			double& xpos,							//!< X position value returned
			double& yscale,						//!< Y scale value returned
			double& yrot,							//!< Y rotation value returned
			double& ypos							//!< Y position value returned
			) const {
			xscale = m_fwd[0][0]; xrot = m_fwd[0][1]; xpos = m_fwd[0][2];
			yscale = m_fwd[1][1]; yrot = m_fwd[1][0]; ypos = m_fwd[1][2];
			}

		//! Determine if tranformation results in orientation (left-handed vs right-handed) reversal.
		bool HasReverseOrientation (
			) const { return ((m_fwd[0][0]*m_fwd[1][1]-m_fwd[0][1]*m_fwd[1][0]) < 0); }

		bool IsEqual (
			const TRANS2D_AFFINE& rhs
			) const {
			return (!memcmp(m_fwd,rhs.m_fwd,sizeof(MAT3X3)) && !memcmp(m_inv,rhs.m_inv,sizeof(MAT3X3)));
			}

		//! Determine if the two transformation's are equivalent
		//! @return 'True' if equivalent, 'false' if not			
		bool IsEquivalentTo (
			const TRANS2D_AFFINE& rhs
			) const {
			for (int i = 0;(i < 3);++i) {
				for (int j = 0;(j < 3);++j) {
					if (fabs(m_fwd[i][j] - rhs.m_fwd[i][j]) > 0.0000001) return (false);
					}
				}
			return (true);
			}

		//! Determine if the transformation is an identity
		//! @return 'True' if identity, 'false' if not			
		bool IsIdentity (
			) const {
			return (m_fwd[0][0] == 1.0 && m_fwd[1][1] == 1.0 && m_fwd[0][1] == 0.0 && m_fwd[0][2] == 0.0 && m_fwd[1][0] == 0.0 && m_fwd[1][2] == 0.0);
			}

		//! Reverse direction of transformation.
		void ReverseDirection (
			) {
			MAT3X3 temp;
			memcpy(temp,m_fwd,sizeof(MAT3X3));
			memcpy(m_fwd,m_inv,sizeof(MAT3X3));
			memcpy(m_inv,temp,sizeof(MAT3X3));
			}

		//! Set transformation from CTRLPOINT array.
		ERRVALUE SetCtrlPoint (
			const CTRLPOINT *cp,					//!< Control point array
			int numpoints,							//!< Number of control points
			bool invert = false					//!< Invert coordinate system (for images)
			) { return (FindBestTrans2DA(numpoints,cp,m_fwd,m_inv,(invert)?TRANS2DMODEL_OrientInvert:0)); }
			
		//! Set transformation from CTRLPOINT3 array.
		ERRVALUE SetCtrlPoint (
			const CTRLPOINT3 *cp,				//!< Control point array
			int numpoints,							//!< Number of control points
			bool invert = false					//!< Invert coordinate system (for images)
			) { return (FindBestTrans2D3A(numpoints,cp,m_fwd,m_inv,(invert)?TRANS2DMODEL_OrientInvert:0)); }
			
		//! Set transformation to "identity".
		void SetIdentity (
			) { trans2dinit(m_fwd,m_inv); }

		//! Set transformation from affine matrices.
		void SetMatrices (
			const MAT3X3 fwd,						//!< Forward transformation
			const MAT3X3 inv						//!< Inverse transformation
			) { memcpy(m_fwd,fwd,sizeof(MAT3X3)); memcpy(m_inv,inv,sizeof(MAT3X3)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MAT3X3 m_fwd;								//!< Forward transformation matrix
		MAT3X3 m_inv;								//!< Inverse transformation matrix

		friend class TRANS2D_MAPGEN;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

inline bool operator== (
	const TRANS2D_AFFINE& lhs,
	const TRANS2D_AFFINE& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const TRANS2D_AFFINE& lhs,
	const TRANS2D_AFFINE& rhs
	) { return (!lhs.IsEqual(rhs)); }


//! Read transformation from ArcWorld file.
//! @return TRUE if successful, FALSE if no file, < 0 if error.
LIBEXPORT int Trans2DReadArcWorld (
	const FILEPATH& filepath,				//!< ArcWorld file path
	TRANS2D_AFFINE& Affine					//!< Transformation returned
	);


#undef LIBEXPORT
#undef CLASSLIBEXPORT

#endif	// INC_MI32_TRANS2DAFFINE_H
