/**
 * \file mi32/vfilters.h
 * \brief Include file for vector filters
 *
 * \if NODOC
 * $Id: vfilters.h_v 1.24 2012/01/06 18:19:31 mju Exp $
 *
 * $Log: vfilters.h_v $
 * Revision 1.24  2012/01/06 18:19:31  mju
 * Remove redundant hdr.
 *
 * Revision 1.23  2007/12/10 21:08:56  scowan
 * Updated parts of this interface.
 *
 * Revision 1.22  2006/10/12 15:45:33  scowan
 * Fwd decl mistring instead of including the file.
 *
 * Revision 1.19  2004/07/29 16:28:31  scowan
 * Added precision filter init function call.
 *
 * Revision 1.6  1998/04/02  21:32:49  scowan
 * Added spline filter.
 *
 * Revision 1.1  1998/03/10  23:19:31  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_VFILTERS_H
#define	INC_MI32_VFILTERS_H

#ifndef	INC_MI32_OBSERVER_H
	#include <mi32/observer.h>
#endif

#ifndef	INC_MI32_ICONID_H
	#include <mi32/iconid.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class VECTORFILTEROBSERVER;
namespace RVC {
	class VECTORTOOLKIT;
	class OBJITEM;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

	
/* Setup and execution structure for all vector filters */
class VECTFILTPARMS : public SUBJECT<VECTORFILTEROBSERVER> {
	public:
		VECTFILTPARMS ();
			
		virtual ~VECTFILTPARMS ();

		//! Perform the filter operation on the vector object provided in the VECTORTOOLKIT instance
		ERRVALUE Execute (
			RVC::VECTORTOOLKIT& vtk
			) { return (v_Execute(vtk)); }
			
		//! Generate the filter parameter form for this filter
		void GenerateForm (
			MDLGPARENT form
			) { v_GenerateForm(form); }
			
		//! Generate a filter result report string.  Only valid after Execute() is called
		void GenerateReportString (
			MISTRING& string
			) const { v_GenerateReportString(string); }
			
		//! Get the label for this filter
		TEXTID GetLabel (
			) const { return (v_GetLabel()); }
			
		//! Get the ICONID for this filter
		ICONID GetIconId (
			) const { return (v_GetIconId()); }
			
		//! Get the HELPID entry for this filter
		HELPID GetHelpID (
			) const { return (v_GetHelpID()); }

		//! Does the filter have a filter parameter form to generate?
		bool HasForm (
			) const { return (v_HasForm()); } 
			
		//! Does the filter operate on polygons?
		bool IsPolygonFilter (
			) const { return (v_IsPolygonFilter()); }
			
		//! Update the object the filter is currently working on
		void UpdateObject (
			const RVC::OBJITEM& ObjItem
			) { v_UpdateObject(ObjItem); }
			
	protected:
	
		#ifndef GENERATING_DOXYGEN_OUTPUT
			void CallOnFilterReady (bool IsReady);
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
			//! Pure virtual methods 
			virtual ERRVALUE v_Execute (RVC::VECTORTOOLKIT& vtk) = 0;
			virtual TEXTID v_GetLabel () const = 0;
			virtual ICONID v_GetIconId () const = 0;

			//! Virtual methods 
			virtual void v_GenerateReportString (MISTRING& string) const;
			virtual void v_GenerateForm (MDLGPARENT form);
			virtual HELPID v_GetHelpID () const;
			virtual bool v_HasForm () const;
			virtual bool v_IsPolygonFilter () const;
			virtual void v_UpdateObject (const RVC::OBJITEM& ObjItem);
		#endif
	};


//! Base Vector Filter observer class.
class VECTORFILTEROBSERVER : public OBSERVER_BASE {
	public:

		explicit VECTORFILTEROBSERVER (
			VECTFILTPARMS& vfp
			);
			
		virtual ~VECTORFILTEROBSERVER (
			);
			
		virtual void OnFilterReady (
			bool IsReady
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		VECTORFILTEROBSERVER (const VECTORFILTEROBSERVER&);
		VECTORFILTEROBSERVER& operator= (const VECTORFILTEROBSERVER&);
		friend class VECTFILTPARMS;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//!\addtogroup VectFilt Vector Filter Functions
//!@{

//! Initialize Dangling Lines Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfDanglingLinesInit (
	);

//! Initialize Dissolve Polys Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfDissolvePolysInit (
	);

//! Initialize Line Thinning Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfLineThinningInit (
	);

//! Initialize Excess Nodes Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfRemoveBubblesInit (
	);

//! Initialize Excess Nodes Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfRemoveExcessNodesInit (
	);

//! Initialize Remove Islands Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfRemoveIslandsInit (
	);

//! Initialize Precision Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfPrecisionInit (
	);
	
//! Initialize Script Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfScriptInit (
	);

//! Initialize Sliver Polygons Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfSliverPolygonsInit (
	);

//! Initialize Spline Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfSplineInit (
	);

//! Initialize Undershoot Filter.
//! @return Interface pointer to filter methods
VECTFILTPARMS* VfUndershootsInit (
	);

//!@}

#endif					//!<  #ifndef INC_MI32_VFILTERS_H 
