/***
 * \file rvc/raster.h
 * \brief interface for the RVC::RASTER class.
 *
 *	\if NODOC
 *	$Id: raster.h_v 1.111 2012/08/15 15:38:04 mju Exp $
 *
 *	$Log: raster.h_v $
 *	Revision 1.111  2012/08/15 15:38:04  mju
 *	Add imagescale::isEqual and getDataUnit...
 *
 *	Revision 1.110  2012/08/14 21:49:59  mju
 *	Add data unit to imagescale.
 *
 *	Revision 1.109  2012/06/08 14:57:57  mju
 *	Add getComponentTextID.
 *
 *	Revision 1.107  2009/03/24 22:14:25  bcolombe
 *	Added AutoLinkJP2
 *
 *	Revision 1.106  2008/05/01 19:58:42  mju
 *	Add getPyramidInfo to return array of tier dimensions.
 *
 *	Revision 1.104  2008/01/09 17:37:49  mju
 *	Remove comptype_none, add "correct" name for comptype_JPEG2000.
 *
 *	Revision 1.103  2007/11/08 18:28:06  scowan
 *	Added method to get the raster components univariate statistics.
 *
 *	Revision 1.102  2007/08/28 16:12:27  mju
 *	Add linkparms_gdal.
 *
 *	Revision 1.100  2007/04/27 20:27:16  scowan
 *	Added static version of getPyramidList.
 *
 *	Revision 1.96  2007/03/13 15:25:02  mju
 *	Change getTileSize to return current settings and make inline.
 *
 *	Revision 1.94  2007/02/21 17:55:42  scowan
 *	Support empty tile compression type.
 *
 *	Revision 1.91  2007/01/12 23:09:09  dfriberg
 *	Allow access of linktype in RASTER::LINKPARMS
 *
 *	Revision 1.90  2006/12/29 00:37:15  dfriberg
 *	Add LINKTYPE_GDAL
 *
 *	Revision 1.89  2006/10/27 22:50:04  scowan
 *	Made getCellType virtual.
 *
 *	Revision 1.86  2006/10/09 14:53:56  dfriberg
 *	Add method for getting embedded link info
 *
 *	Revision 1.82  2006/06/27 15:04:45  scowan
 *	Make method now fills in objitem with created object info.
 *
 *	Revision 1.81  2006/05/31 15:05:57  scowan
 *	Added setCacheRect virtual method.
 *
 *	Revision 1.80  2006/05/05 16:51:37  dfriberg
 *	Add LINKTYPE_Multi
 *
 *	Revision 1.79  2006/04/19 03:38:07  dfriberg
 *	Add Link method that allows settings usage
 *
 *	Revision 1.78  2006/04/12 14:47:06  mju
 *	Un-deprecate makePyramidSet variant which has no compression parameters.
 *
 *	Revision 1.76  2006/03/22 16:09:14  dfriberg
 *	Compression parameters added to MakePyramidSet
 *
 *	Revision 1.73  2005/12/19 17:44:31  dwilliss
 *	adjusted default credentials to account for prior default parameters
 *
 *	Revision 1.72  2005/12/19 17:02:23  linux32build!dwilliss
 *	Credentials can now be made mandatory by setting a #define
 *
 *	Revision 1.70  2005/12/02 20:33:38  mju
 *	Add copyRasterTo methods.
 *
 *	Revision 1.68  2005/11/23 18:37:16  apple64build!build
 *	Created INTERLEAVE_TYPE enum in LINKPARMS_GENERIC
 *
 *	Revision 1.64  2005/10/07 17:13:38  dfriberg
 *	Add ImageScale set method for LINKPARMS_GENERIC
 *
 *	Revision 1.60  2005/09/07 23:00:49  dfriberg
 *	Add optional parameter to CopySampledTo for specifying destination rectangle
 *
 *	Revision 1.59  2005/09/01 22:37:18  scowan
 *	Added queryStepOrder method.
 *
 *	Revision 1.58  2005/08/18 20:52:40  dfriberg
 *	Added CopySampledTo that takes a LRECT2D for copying rectangle
 *
 *	Revision 1.55  2005/04/15 19:36:30  mju
 *	Add parm to fix celltype in setCompression.
 *
 *	Revision 1.54  2005/04/01 15:27:39  mju
 *	Default image line/column 'scale' to 0 so indicates not set.
 *
 *	Revision 1.1  2000/10/02 17:23:21  scowan
 *	Initial revision
 *	\endif
***/

#ifndef INC_RVC_RASTER_H
#define INC_RVC_RASTER_H

#ifndef INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif

#ifndef INC_MI32_ANYRASTV_H
	#include <mi32/anyrastv.h>
#endif

#ifndef INC_MI32_IMAGEDIMENSIONS_H
	#include <mi32/imagedimensions.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class COLOR_ARRAY;
class STAT_UNIVARIATE;
namespace SML {
	class CONTEXT;
	}
namespace RVC {
	class RASTERHANDLE;
	struct RASTERLINKPARMS;
	struct RASTLINKDATAGEN;
	struct RASTINFO;
	}
namespace UOM {
	class UNIT;
	}
#endif // GENERATING_DOXYGEN_OUTPUT


namespace RVC {

//! Manage RVC::OBJTYPE_Raster RVC objects.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_Hyperspectral
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_Thumbnail
class RASTER : public RVC::IMAGE {
	public:

		enum COMPONENT {				//! Possible data conversion modes for Read(), ReadColumn() and Write()
			COMPONENT_Default = 0,		//!< Select default component
			COMPONENT_Red,	 				//!< Convert RGB to Red
			COMPONENT_Green,	 			//!< Convert RGB to Green
			COMPONENT_Blue,	 			//!< Convert RGB to Blue
			COMPONENT_Real,				//!< Convert to Real complex component
			COMPONENT_Imaginary,			//!< Convert to Imaginary complex component
			COMPONENT_Magnitude,			//!< Convert to Magnitude complex component
			COMPONENT_Phase,				//!< Convert to Phase complex component
			COMPONENT_Alpha, 				//!< Convert to Alpha channel component from RGBA or ARGB
			COMPONENT_Cyan, 				//!< Convert to Cyan from CMYK or KCMY
			COMPONENT_Magenta,			//!< Convert to Magenta from CMYK or KCMY
			COMPONENT_Yellow,				//!< Convert to Yellow from CMYK or KCMY
			COMPONENT_Black				//!< Convert to Black from CMYK or KCMY
			};

		enum COMPTYPE {				//! Raster compression types
			COMPTYPE_None =		0,		//!< No compression within each tile (empty tiles not stored though)
			COMPTYPE_RLE =			1,		//!< RLE compression, valid for 1-bit rasters
			COMPTYPE_JPEG =		2,		//!< JPEG compression, valid for 8-bit and 24-bit rasters
			COMPTYPE_Huffman =	3,		//!< Huffman compression, used in Hyperspectral images
			COMPTYPE_DPCM =		4,		//!< DPCM compression
			COMPTYPE_JPEG2000 =	5,		//!< JPEG2000 compression
			COMPTYPE_JP2 =			COMPTYPE_JPEG2000,
			};

		enum LINKTYPE {					//! Type of external objects that can be linked to
			LINKTYPE_NoLink = 0,			//!< Raster is internal
			LINKTYPE_TIFF = 2,			//!< Raster is a link to TIFF image
			LINKTYPE_Generic = 3,		//!< Raster is a link to a generic raster image
			LINKTYPE_MrSID = 4,
			LINKTYPE_ECW = 5,
			LINKTYPE_JP2 = 6,
			LINKTYPE_SocetSetDT = 7,
			LINKTYPE_PNG = 8,
			LINKTYPE_JPEG = 9,
			LINKTYPE_GDAL = 10,
			LINKTYPE_Multi = 2000
			};

		enum STEPORDER {
			STEPORDER_FullSequential,		//!< Full width section sequentially from beginning of image
			STEPORDER_TileSequential,		//!< Tile width section sequentially from beginning of tile
			STEPORDER_TileRandom,			//!< Section anywhere entirely within single tile
			STEPORDER_AnyRandom				//!< Section anywhere in image, even across tile boundaries
			};

		enum USAGE {					//! Raster usage values, used in Make()
			USAGE_Default = 0,			//!< Ordinary raster
			USAGE_HyspcBand,				//!< HyperSpectral Band raster
			USAGE_HyspcClass,				//!< HyperSpectral Class raster
			USAGE_HyspcAverage,			//!< HyperSpectral Average raster
			USAGE_Pyramid,					//!< Pyramid raster
			USAGE_PrinComp,				//!< Principle Components raster
			USAGE_NullBand,				//!< Null band raster
			USAGE_Opacity,					//!< Opacity mask raster
			USAGE_SurfaceElevation,		//!< Surface Elevation raster
			USAGE_SurfaceError,			//!< Surface Error raster
			USAGE_SurfaceRadius,			//!< Surface Radius raster
			USAGE_MultiRasterTile		//!< Tile of a multiraster
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Forward declarations
		class GETOBJFILTER_ISSELECTABLE;
		class GETOBJFILTER_ISOVERLAPPING;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Class to implement raster scale values
		class IMAGESCALE {
			public:

				//! Default constructor.
				IMAGESCALE ();

				//! Obtain raster column scale
				//! @return raster column scale
				double GetColumnScale () const
					{ return (m_ColumnScale);	}

				//! Obtain raster data offset
				//! @return raster data offset
				double GetDataOffset () const
					{ return (m_DataOffset); }

				//! Obtain raster data scale
				//! @return raster data scale
				double GetDataScale () const
					{ return (m_DataScale); }

				//! Get measurement unit for data value (after any scale/offset applied).
				//! @return 'true' if unit defined, 'false' if not.
				bool GetDataUnit (
					UOM::UNIT& unit
					) const;

				//! Get measurement unit number for data value.
				UINT8 GetDataUnitNumber () const
					{ return (m_UnitNumber); }

				//! Get measurement unit type for data value.
				UINT8 GetDataUnitType () const
					{ return (m_UnitType); }

				//! Obtain raster line scale
				//! @return raster line scale
				double GetLineScale () const
					{ return (m_LineScale); }

				//! Determine if equal.
				bool IsEqual (
					const IMAGESCALE& rhs
					) const;

				//! Set raster column scale
				void SetColumnScale (
					double ColumnScale
					) { m_ColumnScale = ColumnScale; }

				//! Set raster data offset
				void SetDataOffset (
					double DataOffset
					) { m_DataOffset = DataOffset;	}

				//! Set raster data scale
				void SetDataScale (
					double DataScale
					) { m_DataScale = (DataScale != 0.0) ? DataScale : 1.0; }

				//! Set measurement unit for stored data values.
				void SetDataUnit (
					const UOM::UNIT& unit
					);

				//! Set measurement unit for stored data values.
				void SetDataUnit (
					UINT8 UnitType,
					UINT8 UnitNumber
					) { m_UnitType = UnitType; m_UnitNumber = UnitNumber; }

				//! Set raster line scale
				void SetLineScale (
					double LineScale
					) { m_LineScale = LineScale; }

				#ifndef GENERATING_DOXYGEN_OUTPUT

				//! \internal
				void GetInRastInfo (
					RASTINFO& rinfo,
					bool IncludeRowCol = true
					) const;

				//! \internal
				void SetFromRastInfo (
					const RASTINFO& rinfo
					);

			private:

				double m_LineScale;				//!< Line scale
				double m_ColumnScale;			//!< Column scale
				double m_DataScale;				//!< Scale factor for cell value
				double m_DataOffset;				//!< Offset for cell value
				UINT8 m_UnitType;
				UINT8 m_UnitNumber;
				#endif
			};	// End IMAGESCALE nested class.

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Forward declarations
		class LINKPARMS;
		class LINKPARMS_ECW;
		class LINKPARMS_GDAL;
		class LINKPARMS_GENERIC;
		class LINKPARMS_JP2;
		class LINKPARMS_JPEG;
		class LINKPARMS_MRSID;
		class LINKPARMS_PNG;
		class LINKPARMS_SOCETSETDT;
		class LINKPARMS_TIFF;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Class to handle values needed to define the characteristics of a raster
		class MAKEPARMS {
			public:

				//! Constructor, initialize with critical raster creation parameters
				MAKEPARMS (
					INT32 NumLines,
					INT32 NumColumns,
					IMAGE::CELLTYPE CellType
					) :
					m_NumLines(NumLines),
					m_NumColumns(NumColumns),
					m_CellType(CellType),
					m_TileLines(0),
					m_TileColumns(0),
					m_Usage(USAGE_Default),
					m_Inverted(false),
					m_PackOrder(PACKORDER_MostSigBit),
					m_CompType(COMPTYPE_None),
					m_CompQuality(75),
					m_UsedBits(0),
					m_IsCompressed(false),
					m_IsLossyCompressed(false),
					m_Fill(false),
					m_NoEmbeddedPyramidTiers(false)
					{ }

				//! Template constructor
				MAKEPARMS (
					const RASTER& Template,				//!< Raster template to determine NumLines, NumColumns, Usage, and scaling
					IMAGE::CELLTYPE CellType = CELLTYPE_Invalid	//!< Optional cell type, default uses type from template
					) :
					m_NumLines(Template.GetNumLines()),
					m_NumColumns(Template.GetNumColumns()),
					m_CellType((CellType!=CELLTYPE_Invalid)?CellType:Template.GetCellType()),
					m_TileLines(Template.GetNumTileLines()),
					m_TileColumns(Template.GetNumTileColumns()),
					m_Usage(Template.GetUsage()),
					m_Inverted(false),
					m_PackOrder(PACKORDER_MostSigBit),
					m_CompType(COMPTYPE_None),
					m_CompQuality(75),
					m_UsedBits(Template.GetNumUsedBits()),
					m_IsCompressed(false),
					m_IsLossyCompressed(false),
					m_Fill(false),
					m_NoEmbeddedPyramidTiers(false)
					{
					Template.GetImageScale(m_ImageScale);
					}

				//! Obtain image cell type.
				//! @return RVC::IMAGE::CELLTYPE for the raster.
				IMAGE::CELLTYPE GetCellType (
					) const { return (m_CellType); }

				//! Obtain raster compression quality
				//! @return Compression quality for the raster.
				UINT16 GetCompressionQuality (
					) const { return (m_CompQuality); }

				//! Obtain raster compression type
				//! @return RVC::RASTER::COMPTYPE for the raster.
				COMPTYPE GetCompressionType (
					) const { return (m_CompType); }

				//! Obtain datatype based on raster creation parameters
				//! @return RVC::IMAGE::DATATYPE of the raster
				IMAGE::DATATYPE GetDataType (
					) const { return (IMAGE::GetDataTypeFromCellType(m_CellType)); }

				//! Obtain default raster tile size based on creation parameters
				//! @return default tile size
				INT32 GetDftTileSize (
					) const;

				//! Get image scale information (RVC::RASTER::IMAGESCALE)
				//! @return IMAGESCALE reference
				const IMAGESCALE& GetImageScale (
					) const { return (m_ImageScale); }

				//! Get initial raster value
				bool GetInitialValue (
					ANYRASTVALUE& value
					) const {
					value = m_InitValue;
					return (m_Fill);
					}

				//! Obtain number of bits in a raster cell
				//! @return number of raster bits
				UINT16 GetNumBits (
					) const { return (IMAGE::GetNumBitsFromCellType(m_CellType)); }

				//! Obtain number of columns in a raster
				//! @return number of raster columns
				INT32 GetNumColumns (
					) const { return (m_NumColumns); }

				//! Obtain number of lines in a raster
				//! @return number of raster lines
				INT32 GetNumLines (
					) const { return (m_NumLines); }

				//! Get the number of used bits.
				//! This related to JP2 compression and is dependent on the source raster value range.
				//! The number of used bits will be less than or equal to the number of bits in the created raster object.
				UINT16 GetNumUsedBits (
					) const { return (m_UsedBits); }

				//! Obtain current raster tile size values.
				void GetTileSize (
					INT32& TileLines,				//!< Number of lines in a tile RETURNED
					INT32& TileColumns			//!< Number of columns in a tile RETURNED
					) const { TileLines = m_TileLines; TileColumns = m_TileColumns; }

				//! Obtain raster value range
				void GetValueMinMax (
					ANYRASTVALUE& min,			//!< Minimum possible raster value
					ANYRASTVALUE& max				//!< Maximum possible raster value
					) const;

				//! Is the raster compressed with a lossy compression algorithm?
				//! @return true if the raster is compressed with a lossy compression algorithm
				bool IsLossyCompressed (
					) const { return (m_IsLossyCompressed); }

				//! Is the compression type valid for the raster cell type
				//! @return True if it is, false if not
				bool IsValidCompression (
					COMPTYPE CompType
					) const;

				//! Set image cell type.
				void SetCellType (
					CELLTYPE CellType
					) { m_CellType = CellType; }

				//! Set raster compression values
				ERRVALUE SetCompression (
					COMPTYPE CompType,			//!< Raster compression type
					UINT16 CompQuality,			//!< Compression quality for lossy compression
					bool FixCellType = false	//!< Change cell type if unsupported by compression type
					);

				//! Set whether to include embedded pyramid tiers if supported.
				//! Used in JPEG2000 compressed rasters produce only single resolution level.
				//! Default is to include embedded pyramid tiers.
				void SetIncludeEmbeddedPyramidTiers (
					bool IncludeTiers
					) { m_NoEmbeddedPyramidTiers = !IncludeTiers; }

				//! Set image scale information (RVC::RASTER::IMAGESCALE)
				void SetImageScale (
					const IMAGESCALE& ImageScale
					) { m_ImageScale = ImageScale;	}

				//! Set initial raster value.
				void SetInitialValue (
					const ANYRASTVALUE& value
					) { m_InitValue = value; m_Fill = true; }

				//! Set raster inversion flag
				void SetInverted (
					) { m_Inverted = true; }

				//! Set number of columns in a raster
				void SetNumColumns (
					INT32 NumColumns
					) { m_NumColumns = NumColumns; }

				//! Set number of lines in a raster
				void SetNumLines (
					INT32 NumLines
					) { m_NumLines = NumLines; }

				//! Set the number of used bits.
				//! This related to JP2 compression and is dependent on the source raster value range.
				//! The number of used bits will be less than or equal to the number of bits in the created raster object.
				void SetNumUsedBits (
					const RASTER& SrcRastObj
					);

				//! Set the number of used bits.
				//! This related to JP2 compression and is dependent on the source raster value range.
				//! The number of used bits will be less than or equal to the number of bits in the created raster object.
				void SetNumUsedBits (
					UINT16 UsedBits
					) { m_UsedBits = UsedBits; }

				//! Set raster pack order
				void SetPackOrder (
					PACKORDER PackOrder			//!< New raster pack order
					) { m_PackOrder = PackOrder; }

				//! Set raster tile size.
				void SetTileSize (
					INT32 TileLines,
					INT32 TileColumns
					) { m_TileLines = TileLines; m_TileColumns = TileColumns; }

				//! Set raster usage parameter.
				void SetUsage (
					USAGE usage
					) { m_Usage = usage; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				INT32 m_NumLines;					//!< Number of lines in raster
				INT32 m_NumColumns;				//!< Number of columns in raster
				CELLTYPE m_CellType;				//!< Image cell type
				PACKORDER m_PackOrder;			//!< Raster pack order
				USAGE m_Usage;						//!< Raster usage, see enum
				INT32 m_TileLines;				//!< Number of lines per tile
				INT32 m_TileColumns;				//!< Number of columns per tile
				COMPTYPE m_CompType;				//!< Compression type
				UINT16 m_CompQuality;			//!< Compression quality, 1-100, 0 == default
				UINT16 m_UsedBits;				//!< The number of used bits will be less than or equal to the number of bits in the created raster object.
				bool m_Inverted;					//!< Is raster inverted?
				bool m_IsCompressed;				//!< Is raster compressed
				bool m_IsLossyCompressed;		//!< Is raster lossy compressed
				bool m_Fill;
				bool m_NoEmbeddedPyramidTiers;
				ANYRASTVALUE m_InitValue;		//!< Value to initialize raster with
				IMAGESCALE m_ImageScale;

				friend class RASTER;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of MAKEPARMS nested class.

		static ERRVALUE AutoLinkJP2 (
			RVC::OBJECT& ParentObj,
			const FILEPATH& LinkFile,
			RVC::GEOSPATIAL::MAKELINKFLAGS LinkFlags
			);

		static ERRVALUE AutoLinkTIFF (
			RVC::OBJECT& ParentObj,
			const FILEPATH& LinkFile,
			RVC::GEOSPATIAL::MAKELINKFLAGS LinkFlags
			);

		//! Compute the next pyramid size based on the sizes passed in
		//! @return 'True' if the next size is valid, 'false' if not
		static bool ComputeNextPyramidSize (
			INT32& NumColumns,
			INT32& NumLines,
			bool IsFirst								//!< True if for first tier, false for additional tiers
			);

		//! Get TEXTID for name of COMPONENT.
		static TEXTID GetComponentTextID (
			COMPONENT component
			);

		//! Return list of available raster pyramid sub-rasters.
		//!	The TierInfo array will be sorted in ascending order by sampling and the array and list are aligned.
		static ERRVALUE GetPyramidInfo (
			const OBJITEM& RastObjItem,
			SIMPLE_ARRAY<IMAGEDIMENSIONS>& TierInfo,						//!< Simple array of tier dimensions and sampling returned.
			RVC::OBJITEMLIST& PyramidList,									//!< List of pyramid raster ObjItem's RETURNED
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Return list of available raster pyramid sub-rasters.
		//!	The sample rate list will be sorted in ascending order and the array and list are aligned.
		static ERRVALUE GetPyramidList (
			const OBJITEM& RastObjItem,
			SIMPLE_ARRAY<INT32>& SampleList,									//!< Simple array of available sample rates RETURNED
			RVC::OBJITEMLIST& PyramidList,									//!< List of pyramid raster ObjItem's RETURNED
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Default constructor.
		RASTER ();

		//! Copy constructor.
		RASTER (
			const RVC::RASTER& rhs
			);

		//! Destructor
		virtual ~RASTER ();

		//! Assignment.
		RVC::RASTER& operator= (
			const RVC::RASTER& rhs
			);

		//! Compute Univariate Statistics on this raster object
		ERRVALUE ComputeUnivariateStatistics (
			STAT_UNIVARIATE& StatUnivariate,				//!< Univariate Statistics RETURNED
			COMPONENT Component = COMPONENT_Default	//!< Component extraction value
			);

		//! Copy the raster into the 'DestRaster' object using the sampling rates 'ColumnSample' and 'LineSample'
		//! This algorithm averages the cell cluster based on the sample rate for the destination
		ERRVALUE CopyAverageTo (
			RASTER& DestRaster,
			INT32 ColumnSample,
			INT32 LineSample,
			COLOR_ARRAY* ColorMap = 0
			);

		//! Copy the raster to the specified target location.
		//! The target raster will be created with the same dimensions as the raster being copied.
		//! If no compatible cell type is specified in the MakeParm instance, the cell type will be
		//! determined from the source raster.  The cell type may also be adjusted based on the compression settings.
		//! Use of this function may require computation of a histogram for the source raster and may also
		//! require multiple processing passes on the source raster.
		ERRVALUE CopyRasterTo (
			RVC::OBJITEM& ObjItemTarget,			//!< Target raster location passed/returned
			const MAKEPARMS& MakeParm,				//!< Parameters for target raster creation
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Copy the raster into the 'TargetRaster' object.
		//! The target raster must have the same dimensions and usually the same cell type as the raster being
		//! copied, however, limited cell type conversions where required for compression are supported.
		ERRVALUE CopyRasterTo (
			RASTER& TargetRaster						//! Newly created raster to copy into
			);

		//! Copy the raster into the 'DestRaster' object using the sampling rates 'ColumnSample' and 'LineSample'
		//! This algorithm selects a cell to copy to the destination raster based on the sample rate
		ERRVALUE CopySampledTo (
			RASTER& DestRaster,
			INT32 ColumnSample,
			INT32 LineSample
			);

		//! Copy the raster into the 'DestRaster' object using the sampling rates 'ColumnSample' and 'LineSample'
		//! This algorithm selects a cell to copy to the destination raster based on the sample rate
		//!
		//! When constructing your ExtractRect (and optional DestRect), note that this method considers the
		//! "initial" x and y coordinates to be the upper-left hand corner of the image, and the "last" coordinates
		//! to be the lower-right hand corner.
		ERRVALUE CopySampledTo (
			RASTER& DestRaster,
			INT32 ColumnSample,
			INT32 LineSample,
			const LRECT2D& ExtractRect,		//! Initial / last rows and columns to copy to DestRaster.
			LRECT2D* DestRect = 0				//! Destination position.  Only need to supply start line and column coordinates, final x and y will be filled in by this method.
			);

		//! Create an SML context for the given raster class instance
		ERRVALUE CreateSMLContext (
			SML::CONTEXT **pcontext					//!< SML context allocated and RETURNED
			);

		//! Extract portion of raster, including null value/mask and georeference.
		ERRVALUE ExtractTo (
			RASTER& TgtRaster,					//! Destination raster
			const LRECT2D& SrcRect,				//! Source rectangle to extract
			LRECT2D* TgtRect = 0					//! Destination position.  Only need to supply start line and column coordinates, final x and y will be filled in by this method.
			);

		//! Obtain array of components present for this raster.
		ERRVALUE GetComponents (
			SIMPLE_ARRAY<COMPONENT>& Components
			) const;

		//! Obtain raster compression type
		//! @return RVC::RASTER::COMPTYPE for the raster.
		COMPTYPE GetCompressionType (
			) const { return (m_MakeParms.m_CompType); }

		//! Obtain raster datatype based on RVC::IMAGE::DATATYPE enumeration
		//! @return RVC::IMAGE::DATATYPE of the raster
		IMAGE::DATATYPE GetDataType (
			) const { return (m_MakeParms.GetDataType()); }

		//! Linked rasters only - finds sampling information for embedded reduced 
		//! resolution image data if the link format supports it. 
		ERRVALUE GetEmbeddedPyramidList (
			SIMPLE_ARRAY<INT32>& samplerates
			);

		//! Obtain raster scale values (line, column, data scale and data offset)
		ERRVALUE GetImageScale (
			IMAGESCALE& scale						//!< Class to hold image scale values RETURNED
			) const;

		//! Obtain type of external image the raster object is linked to
		//! @return RVC::RASTER::LINKTYPE enumeration value
		LINKTYPE GetLinkType (
			) const { return (m_LinkType); }

		//! Obtain raster creation parameters
		//! @return const RVC::RASTER::MAKEPARMS
		const MAKEPARMS& GetMakeParms (
			) const { return (m_MakeParms); }

		//! Obtain raster null value information
		ERRVALUE GetNullValue (
			NULLVALUE& NullValue					//!< Current NULLVALUE to get from raster
			) const;

		//! Obtain number of bits in a raster cell
		//! @return number of bits in a raster cell
		UINT16 GetNumBits (
			) const { return (m_MakeParms.GetNumBits()); }

		//! Obtain number of columns in a raster tile
		//! @return number of columns in the raster tile
		INT32 GetNumTileColumns (
			) const { return (m_MakeParms.m_TileColumns); }

		//! Obtain number of lines in a raster tile
		//! @return number of lines in the raster tile
		INT32 GetNumTileLines (
			) const { return (m_MakeParms.m_TileLines); }

		//! Get the number of used bits.
		//! This related to JP2 compression and is dependent on the source raster value range.
		//! The number of used bits will be less than or equal to the number of bits in the created raster object.
		UINT16 GetNumUsedBits (
			) const { return (m_MakeParms.m_UsedBits); }

		//! Obtain number of tiles in a raster row
		//! @return number of row tiles
		INT32 GetNumXTiles (
			) const { return (m_XTiles); }

		//! Obtain number of tiles in a raster column
		//! @return number of column tiles
		INT32 GetNumYTiles (
			) const { return (m_YTiles); }

		//! Obtain raster pack order
		//! @return Raster pack order
		PACKORDER GetPackOrder (
			) const { return (m_MakeParms.m_PackOrder); }

		//! Return list of available raster pyramid sub-rasters.
		//!	The TierInfo array will be sorted in ascending order by sampling and the array and list are aligned.
		ERRVALUE GetPyramidInfo (
			SIMPLE_ARRAY<IMAGEDIMENSIONS>& TierInfo,	//!< Simple array of tier dimensions and sampling returned.
			RVC::OBJITEMLIST& PyramidList					//!< List of pyramid raster ObjItem's RETURNED
			) const;

		//! Return list of available raster pyramid sub-rasters.
		//!	The sample rate list will be sorted in ascending order and the array and list are aligned.
		ERRVALUE GetPyramidList (
			SIMPLE_ARRAY<INT32>& SampleList,		//!< Simple array of available sample rates RETURNED
			RVC::OBJITEMLIST& PyramidList			//!< List of pyramid raster ObjItem's RETURNED
			) const;

		//! Obtain number of bytes in a raster tile
		//! @return size of raster tile in bytes
		INT32 GetTileSizeBytes (
			) const { return (m_TileSize); }

		//! Obtain purpose of raster image
		//! @return RVC::RASTER::USAGE enumeration value
		USAGE GetUsage (
			) const { return (m_MakeParms.m_Usage); }

		//! Determine if null value exists for raster.
		bool HasNullValue () const;

		//! Is the raster compressed?
		//! @return true if the raster is compressed
		bool IsCompressed (
			) const { return (m_MakeParms.m_IsCompressed); }

		//! Is the raster compressed with a lossy compression algorithm?
		//! @return true if the raster is compressed with a lossy compression algorithm
		bool IsLossyCompressed (
			) const { return (m_MakeParms.m_IsLossyCompressed); }

		//! Is the raster flipped around such that the first raster line is at the bottom of the image
		//! @return true if the raster is inverted
		bool IsInverted (
			) const { return (m_MakeParms.m_Inverted); }

		//! Create a raster object and establish a link to an external image
		ERRVALUE Link (
	    	const RVC::OBJECT& parent,					//!< Parent object to create raster under
			const RVC::DESCRIPTOR& Descriptor,
			const LINKPARMS& LinkParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a raster object and establish a link to an external image
		ERRVALUE Link (
	    	const RVC::OBJECT& parent,					//!< Parent object to create raster under
			const RVC::DESCRIPTOR& Descriptor,
			const LINKPARMS& LinkParms,
			USAGE Usage,																		//!< raster object created will have this usage
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a raster object
		ERRVALUE Make (
	    	const RVC::OBJECT& parent,					//!< Parent object to create raster under
			const RVC::DESCRIPTOR& Descriptor,
			MAKEPARMS& MakeParms,							//!< Parameters needed to create raster, cannot be changed after creation
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a raster object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			MAKEPARMS& MakeParms,					//!< Parameters needed to create raster, cannot be changed after creation
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,						//!< Source of raster creation, default is GetAppName()
#else
			const char* Source = 0,						//!< Source of raster creation, default is GetAppName()
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create pyramid set under this raster object
		//! It will not write over valid pyramids unless PYRAMID_Rebuild is specified
		ERRVALUE MakePyramidSet (
#ifdef RVC_CREDENTIALS_MANDATORY
			IMAGE::PYRAMID flags,
#else
			IMAGE::PYRAMID flags = IMAGE::PYRAMID_Default,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		ERRVALUE MakePyramidSet (
#ifdef RVC_CREDENTIALS_MANDATORY
			RASTER::COMPTYPE comptype,
			UINT16 compquality,
			IMAGE::PYRAMID flags,
#else
			RASTER::COMPTYPE comptype,
			UINT16 compquality,
			IMAGE::PYRAMID flags = IMAGE::PYRAMID_Default,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Query support/efficiency for specified stepping order.
		//! @return Efficiency between 0 and 1, 0 indicates not supported, 1 indicates most efficient.
		float QueryStepOrder (
			STEPORDER steporder						//!< Stepping order to query
			);

		//! Read a line of raster cells up to a certain length
		ERRVALUE Read (
			INT32 Line,											//!< Raster line to read
			INT32 StartColumn,								//!< Raster column to start reading on
			void *data,											//!< Buffer to read into
			INT32 Length = 1,									//!< Number of columns to read
			CONVMODE Convert = CONVMODE_None,			//!< Data conversion mode
			COMPONENT Component = COMPONENT_Default	//!< Component extraction value
			) const;

		//! Read a line of raster cells up to a certain length, converting to signed 8 bit
		ERRVALUE Read (
			INT32 Line,											//!< Raster line to read
			INT32 StartColumn,								//!< Raster column to start reading on
			INT8 *data,											//!< Buffer to read into
			INT32 Length = 1,									//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default	//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 8 bit
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT8 *data, 								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Read a line of raster cells up to a certain length, converting to signed 16 bit
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			INT16 *data, 								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 16 bit
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT16 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Read a line of raster cells up to a certain length, converting to signed 32 bit
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			INT32 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 32 bit
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT32 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Read a line of raster cells up to a certain length, converting to 32 bit float
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			float *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_Float, Component)); }

		//! Read a line of raster cells up to a certain length, converting to 64 bit double
		ERRVALUE Read (
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			double *data, 								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (Read(Line, StartColumn, data, Length, CONVMODE_Double, Component)); }

		//! Read a column of raster cells up to a certain length
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			void *data,									//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			CONVMODE Convert = CONVMODE_None,			//!< Data conversion mode
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const;

		//! Read a column of raster cells up to a certain length, converting to signed 8 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			INT8 *data,									//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Read a column of raster cells up to a certain length, converting to unsigned 8 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			UINT8 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Read a column of raster cells up to a certain length, converting to signed 16 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			INT16 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Read a column of raster cells up to a certain length, converting to unsigned 16 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			UINT16 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Read a column of raster cells up to a certain length, converting to signed 32 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			INT32 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Read a column of raster cells up to a certain length, converting to unsigned 32 bit
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			UINT32 *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Read a column of raster cells up to a certain length, converting to 32 bit float
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			float *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_Float, Component)); }

		//! Read a column of raster cells up to a certain length, converting to 64 bit double
		ERRVALUE ReadColumn (
			INT32 Line,									//!< Raster line to start reading on
			INT32 StartColumn,							//!< Raster column to read
			double *data,								//!< Buffer to read into
			INT32 Length = 1,							//!< Number of lines to read
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) const { return (ReadColumn(Line, StartColumn, data, Length, CONVMODE_Double, Component)); }

		//! Set the raster scale value parameters
		ERRVALUE SetImageScale (
			const IMAGESCALE& scale						//!< Scale values to set in raster
			);

		//! Set the raster null value parameters
		ERRVALUE SetNullValue (
			const NULLVALUE& NullValue,					//!< Null value to set in raster
			bool NullUsed = true								//!< Whether null value is used or not
			);

		//! Write a line of raster cells up to a certain length
		ERRVALUE Write (
			INT32 Line,											//!< Raster line to write
			INT32 StartColumn,								//!< Raster column to start writing on
			const void *data,									//!< Buffer to write from
			INT32 Length = 1,									//!< Number of columns to write
			CONVMODE Convert = CONVMODE_None,			//!< Data conversion mode
			COMPONENT Component = COMPONENT_Default	//!< Component extraction value
			);

		//! Write a line of raster cells up to a certain length, converting from signed 8 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const INT8 *data,									//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Write a line of raster cells up to a certain length, converting from unsigned 8 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const UINT8 *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT8, Component)); }

		//! Write a line of raster cells up to a certain length, converting from signed 16 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const INT16 *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Write a line of raster cells up to a certain length, converting from unsigned 16 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const UINT16 *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT16, Component)); }

		//! Write a line of raster cells up to a certain length, converting from signed 32 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const INT32 *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Write a line of raster cells up to a certain length, converting from unsigned 32 bit
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const UINT32 *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_INT32, Component)); }

		//! Write a line of raster cells up to a certain length, converting from 32 bit float
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const float *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_Float, Component)); }

		//! Write a line of raster cells up to a certain length, converting from 64 bit double
		ERRVALUE Write (
			INT32 Line,									//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const double *data,								//!< Buffer to write from
			INT32 Length = 1,							//!< Number of columns to write
			COMPONENT Component = COMPONENT_Default		//!< Component extraction value
			) { return (Write(Line, StartColumn, data, Length, CONVMODE_Double, Component)); }

	protected:

		void AssignValues (
			const RASTINFO& rinfo
			);

		bool IsRasterHandleValid (
			) { return m_RasterHandle != NULL; }

		void SetRasterHandle (
			RASTERHANDLE* RastHandle
			);

		RASTERHANDLE& GetRasterHandle (
			) { return *m_RasterHandle; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class MAKEPARMS;

		//! Open raster handle, might change later
		RASTERHANDLE *m_RasterHandle;

		//! Unchangeable values, safe from multi-threading
		MAKEPARMS m_MakeParms;			// Raster creation parameters
		LINKTYPE m_LinkType;				// Raster Link Type, see enum
		INT32 m_XTiles;					// Number of XTiles in a raster
		INT32 m_YTiles;					// Number of YTiles in a raster
		INT32 m_TileSize;					// Size of raster tile in bytes

		int ComputeNumPyramidLevels (PYRAMID flags);
		PYRAMID ValidatePyramidFlags (PYRAMID flags) const;

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZOffset () const;
		virtual double v_GetZScale () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZOffset (double zoffset);
		virtual void v_SetZScale (double zscale);

		//! Implementation of RVC::IMAGE virtual methods
		virtual ERRVALUE v_GetBandParms (INT32 BandNum, BANDPARMS& BandParms) const;
		virtual CELLTYPE v_GetCellType () const;
		virtual double v_GetColumnScale () const;
		virtual double v_GetLineScale () const;
		virtual ERRVALUE v_GetNullValue (INT32 BandNum, NULLVALUE& NullValue) const;
		virtual INT32 v_GetNumBands () const;
		virtual INT32 v_GetNumColumns () const;
		virtual INT32 v_GetNumLines () const;
		virtual bool v_HasNullValue (INT32 BandNum) const;
		virtual ERRVALUE v_ReadBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_ReadColumnBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_SetCacheRect (const LRECT2D& Rect);
		virtual ERRVALUE v_SetCacheSize (INT32 NumTiles);
		virtual ERRVALUE v_SetCacheSize (CACHEMODE CacheFlags);
		virtual ERRVALUE v_SetNullValue (INT32 BandNum, const NULLVALUE* NullValue);
		virtual ERRVALUE v_WriteBand (INT32 Band, INT32 Line, INT32 StartColumn, const void *data, INT32 Length, CONVMODE Convert);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Convenience filter for raster selection.
//! This class provides for allowed and/or denied cell types to be specified during
//! construction, as well as whether all rasters in a multiple selection are
//! required to have the same dimensions.  Appropriate message IDs are set
//! based on the specified cell types.
class RASTER::GETOBJFILTER_ISSELECTABLE : public GETOBJFILTER {
	public:

		//! Constructor specifying allowed or denied cell types.
		GETOBJFILTER_ISSELECTABLE (
			RVC::IMAGE::CELLTYPE CellTypes,
			bool DenyTypes = false,
			bool NeedSameSize = false
			);

		virtual ~GETOBJFILTER_ISSELECTABLE () {}

	protected:

		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position = -1);
		virtual bool v_IsValidList (const RVC::OBJITEMLIST& objlist);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		IMAGE::CELLTYPE m_CellTypes;
		bool m_NeedSameSize;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Convenience filter for raster selection.
//! This class combines the RVC::RASTER::GETOBJFILTER_ISSELECTABLE and
//! RVC::GEOSPATIAL::GETOBJFILTER_ISOVERLAPPING filters
class RASTER::GETOBJFILTER_ISOVERLAPPING : public GETOBJFILTER {
	public:

		//! Constructor specifying allowed or denied cell types.
		GETOBJFILTER_ISOVERLAPPING (
			const RVC::OBJITEM& SrcObjItem,
			RVC::IMAGE::CELLTYPE CellTypes,
			bool DenyTypes = false,
			bool NeedSameSize = false
			);

		virtual ~GETOBJFILTER_ISOVERLAPPING () {}

	protected:

		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position = -1);
		virtual bool v_IsValidList (const RVC::OBJITEMLIST& objlist);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RASTER::GETOBJFILTER_ISSELECTABLE m_IsSelectable;
		GEOSPATIAL::GETOBJFILTER_ISOVERLAPPING m_IsOverlapping;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Parmeters needed to link to external rasters
class RASTER::LINKPARMS {
	public:
		virtual ~LINKPARMS (
			);

	RASTER::LINKTYPE GetLinkType() const;

	protected:

		//! Constructor
		LINKPARMS (
			const FILEPATH& filename,				//!< File to link to
			RASTER::LINKTYPE LinkType,				//!< Type of raster link to create
			GEOSPATIAL::MAKELINKFLAGS flags
			);

		RASTLINKDATAGEN& GetGenericLinkData (
			);

		RASTINFO& GetRasterHeader (
			);

		void SetBandList (
			const INT32* BandList,
			INT32 NumBands
			);

		void SetComponent (
			INT32 component
			);

		void SetGenericLinkData (
			const RASTLINKDATAGEN& GenLinkData
			);

		void SetPyramidLink (
			);

		void SetRasterHeader (
			const RASTINFO& RastInfo
			);

		void SetSample (
			INT32 Sample
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RASTERLINKPARMS* m_LinkParms;

		friend class RASTER;
		friend class MULTIRASTER;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};	// End of LINKPARMS nested class.


class RASTER::LINKPARMS_ECW : public RASTER::LINKPARMS {
	public:

		//! Constructor
		//! If the 'BandList' parameter is used it must contain 3 items.
		//! For RGB, each entry should contain the appropriate 0-based band index.
		//! For grayscale, the first entry must contain the band index and the remaining entries must contain -1.
		LINKPARMS_ECW (
			const FILEPATH& filename,				//!< File to link to
			const INT32* BandList,					//!< List of bands, 0 for default
			INT32 Sample = 1,							//!< Sample, 1 unless pyramid tier
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_ECW, flags)
			{
			if (BandList != 0) SetBandList(BandList, 3);
			SetSample(Sample);
			}
	};


class RASTER::LINKPARMS_GDAL : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_GDAL (
			const FILEPATH& filename,				//!< File to link to
			INT32 component = 0,						//!< Component to link to, -1 for RGB, 0 for first component, 1 for second, etc.
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_GDAL, flags)
			{
			SetComponent(component);
			}
	};


class RASTER::LINKPARMS_GENERIC : public RASTER::LINKPARMS {
	public:
		enum LINKORDER {
			LINKORDER_HiLo,
			LINKORDER_LoHi
			};

		enum INTERLEAVE_TYPE {
			INTTYPE_Line,		//Not INTERLEAVE_Line because rvcrast.h has it defined already
			INTTYPE_Band,
			INTTYPE_Pixel
			};

		//! Constructor
		LINKPARMS_GENERIC (
			const FILEPATH& filename,				//!< File to link to
			RVC::IMAGE::CELLTYPE celltype,
			INT32 numlins,
			INT32 numcols,
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			);

		//! Set number of bytes trailing band.
		void SetBandTrailerBytes (
			INT32 bandtbytes
			);

		//! Set byte order of each cell
		void SetByteOrder (
			LINKORDER order
			);

		//! Set the line / column / data scale, and data offset of the image
		void SetImageScale (
			const RASTER::IMAGESCALE& scale
			);

		//! Set type of compression used in imagery
		void SetCompressionType (
			RVC::RASTER::COMPTYPE comptype
			);

		//! Set number of header bytes for band.
		void SetHeaderBytes (
			INT64 headerbytes
			);

		//! Set value to add to line / column before using interleave factor
		void SetInterleaveAdd (
			INT16 intadd
			);

		//! Set interleave trailer in bytes (between lines of same band).
		void SetInterleaveTrailerBytes (
			INT32 intertbytes
			);

		//! Set interleave factor
		void SetInterleaveFactor (
			INT16 intfact
			);

		//! Set interleave type
		void SetInterleaveType (
			INTERLEAVE_TYPE inttype
			);

		//! Set whether the image is inverted
		void SetInverted (
			bool inverted
			);

		//! Sets length of line trailer in bytes
		void SetLineTrailerBytes (
			INT32 linetbytes
			);

		//! Sets the null value of the image
		void SetNullValue (
			const IMAGE::NULLVALUE& nullval
			);

		//! Sets the number of bands (needed if interleaved)
		void SetNumBands (
			INT32 numbands
			);

		//! Sets number of columns per tile
		void SetTileColumns (
			INT32 tilecols
			);

		//! Sets number of lines per tile
		void SetTileLines (
			INT32 tilelins
			);
	};


class RASTER::LINKPARMS_JP2 : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_JP2 (
			const FILEPATH& filename,				//!< File to link to
			INT32 component = 0,						//!< Component to link to, -1 for RGB, 0 for first component, 1 for second, etc.
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_JP2, flags)
			{
			SetComponent(component);
			}

		void SetPyramid (
			) { SetPyramidLink(); }
	};


class RASTER::LINKPARMS_JPEG : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_JPEG (
			const FILEPATH& filename,				//!< File to link to
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_JPEG, flags)
			{}
	};


class RASTER::LINKPARMS_MRSID : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_MRSID (
			const FILEPATH& filename,				//!< File to link to
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_MrSID, flags)
			{}
	};


class RASTER::LINKPARMS_PNG : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_PNG (
			const FILEPATH& filename,				//!< File to link to
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_PNG, flags)
			{}
	};


class RASTER::LINKPARMS_SOCETSETDT : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_SOCETSETDT (
			const FILEPATH& filename,				//!< File to link to
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_SocetSetDT, flags)
			{}
	};


class RASTER::LINKPARMS_TIFF : public RASTER::LINKPARMS {
	public:

		//! Constructor
		LINKPARMS_TIFF (
			const FILEPATH& filename,				//!< File to link to
			INT32 ImageIndex,							//!< Index of image to link to, -1 for default, 0 for first image
			INT32 component = 0,						//!< Component to link to, -1 for RGB
			GEOSPATIAL::MAKELINKFLAGS flags = GEOSPATIAL::MAKELINKFLAG_None
			) :
			LINKPARMS(filename, RASTER::LINKTYPE_TIFF, flags)
			{
			SetComponent(component);
			SetSample(ImageIndex);
			}
	};

}		//! End of RVC namespace

#endif 		//!< INC_RVC_RASTER_H
