/******************************************************************************
 *
 * \file sml/function.h
 * \brief SML FUNCTION
 *
 * \if NODOC
 * $Log: function.h_v $
 * Revision 1.7  2011/10/11 21:13:47  mju
 * Remove redundant hdr.
 *
 * Revision 1.6  2011/08/19 22:42:23  bcolombe
 * Add SMLFUNC_Required
 *
 * Revision 1.5  2008/09/08 13:36:39  mju
 * More doxygen warnings.
 *
 * Revision 1.3  2008/02/15 16:35:42  mju
 * Don't use legacy smlcontext type.
 *
 * Revision 1.2  2007/05/08 15:45:54  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 23:02:44  dwilliss
 * Initial revision
 *
 * \endif
*****************************************************************************/

#ifndef INC_SML_FUNCTION_H
#define INC_SML_FUNCTION_H

#ifndef	INC_SML_FUNCSIG
#include <sml/funcsig.h>
#endif

#ifndef INC_SML_ARG_H
#include <sml/arg.h>
#endif

#ifndef INC_SML_PARM_H
#include <sml/parm.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifdef WIN32
#pragma warning (disable: 4018) /* '!=' : signed/unsigned mismatch */
#pragma warning (disable: 4251) //! struct needs dll-interface blah blah blah
#endif

//! Use this for the desciption field of any member or function that you want
//! the documentation to show as being Deprecated.
//! The last two digits are the version in which the feature will no longer
//! be guarenteed to be available.
#define SML_RETIRE_IN_68	"RetireIn6.8"
#define SML_RETIRE_IN_69	"RetireIn6.9"


namespace SML {
class CONTEXT;
}

/*****************************************************************************/
/**
*	Use this to typecast your function pointer in the SMLFUNC strucure.
* The actual function should return either void, double or (SMLSTRING *),
* depending on the value of type.  If you don't typecast it, the compiler
* will hate you.
*/
typedef void (*VOIDFUNC)(int, SMLARG *, UINT8*, SML::CONTEXT *);
#ifndef GENERATING_DOXYGEN_OUTPUT
class SOMECLASS;
#endif // GENERATING_DOXYGEN_OUTPUT
typedef void (SOMECLASS::*METHODPTR)();

namespace SML {
class CLASSHASH;


//! Possible flags for SMLFUNC.flags
#define SMLFUNC_NotInWin32	0x00000001
#define SMLFUNC_NotInX		0x00000002
#define SMLFUNC_IsMemberFunc	0x00000004
#define SMLFUNC_UseSignature	0x00000008
#define SMLFUNC_HasProtoParms	0x00000010
#define SMLFUNC_Prototype		0x00000020
#define SMLFUNC_Deprecated		0x00000040
#define SMLFUNC_Required		0x00000080

/*****************************************************************************/
//!
//!	Structure for defining an SML function.
//!
//! You don't normally define one of these directly.  Instead, you use macros to
//! define them.
//!
//!	<b>Example:</b>
//!
//! In the following code, "MyFunction" is the name the function will show to SML
//! users.  There are several things you need to define using names based on the
//!	function name and a fixed suffix.  Note, that the descriptions you give here
//!	are used to create smlfuncs.txt, which can then be translated to other
//!	lanugages.
//!
//!	@code
//!		// An example SML::FLAG so we can show how to create a flags parameter
//!		// See SML::FLAG for details.  (You don't have to have one of these)
//!		static SML::FLAG MyFunction_Flags[] = {
//!			{ "flag1",  1, "Description of flag1" },
//!			{ "flag2",  2, "Description of flag2" },
//!			{ NULL }		// Very important!
//!			};
//!
//!		static SML::PARM MyFunction_Parms[] = {
//!		   { "parm1",   STYPE_String, 0, "Describe the parm here", NULL, NULL },
//!			{ "color",   STYPE_Class,  0, "a color", NULL, "COLOR" },
//!			{ "flags",   STYPE_String, PARM_Optional, "describe flags here", MyFunction_Flags, NULL },
//!			{ NULL }		// Very important!
//!			};
//!
//!		#define MyFunction_FuncType    STYPE_BFunction
//!		#define MyFunction_CreateDate  20000517	// YYYYMMDD format
//!		#define MyFunction_ModDate     0          // Havn't changed it yet
//!    // Note, if the function returns a class, you would also have...
//!		#define MyFunction_RetVal      "COLOR"   // The name of the class we return
//!
//!		static double MyFunction (int numargs, SML::ARG *args, UINT8 *type, SML::CONTEXT *context) {
//!			...
//!			return(something);
//!			}
//!	@endcode
//!
//!	Then later, you have...
//!
//!	@code
//!		static SML::FUNC MyFunctionList[] = {
//!		   SMLFUNCDEF2 (MyFunction),
//!			{ NULL }		// Very important!
//!			};
//!
//!		...
//!
//!		context->InstallFunctions("My Function Group", MyFunctionList);
//!	@endcode
//!
//!
//!	There is more documentation for creating function in sml.h but not yet readable through
//!	the Doxygen-created documentation.
//!
class RVCSYSLIBCLASSEXPORT FUNC {
	public:

		const char *name;
		STYPE type;
		VOIDFUNC func;
		PARM *parms;
		const char *ClassName;
		INT32 CreateDate, ModDate;	//!<  Date in format YYYYMMDD eg: 19980204
		int minparms, maxparms;
		const char *description;			//!<  Default used to create smlfuncs.txt
		int sizeofSMLFUNC;
		int sizeofSMLPARM;
		UINT32 flags;
		SMLFUNCSIG signature;
		METHODPTR method;			//!< Note: sizeof(void (SOMECLASS::*METHODPTR)()) varies by system, 16 bytes on Windows, 8 on Linux

		int GetFuncGroupNum (
			) const;

		//! For functions which take a pointer to a function as a parameter,
		//! (eg, to set a callback), this function will (maybe) return an
		//! SML::PARM array of the parameters that the callback will get.
		//! To set this up the parms member of the SML::FUNC will end in an
		//! SML::PARM with a NULL name as usual, but is then followed by the
		//! parms for the callback.  If we've set them up this way, we also
		//! set the  flag SML::FUNC_HasProtoParms so that the documentation
		//! code will know to look for it.
		const PARM* GetCallbackParms (
			) const;


#ifndef GENERATING_DOXYGEN_OUTPUT
		//! I'd make m_funcgroup private but then you can't statically initialize
		//! them the way we do everywhere.
		int m_funcgroup;				//!< FuncGroup number - Note: use method to get
#endif // GENERATING_DOXYGEN_OUTPUT
	//!
	};


//! Structure for defining SML functions.
//! Constructors in this class fill in the SMLFUNC members for you
class FUNCDEF : public FUNC {
	public:

	#ifndef NO_DEPRECATED
	//! Construct an SMLFUNCDEF for a void function
	//! \deprecated Use the version that doesn't take createdate, modifydate
	//! or description parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or description parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		void (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		int  p_createdate, 
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, p_bInWin32);
		type = STYPE_BProcedure;
		func = p_func;
		}
	#endif

//#ifndef UNIX
// Go ahead and allow it. We can fix whatever doesn't compile to not use the
// deprecated version
	// For now, we can't have this until we fix a few places that pass create date
	//! Construct an SML::FUNCDEF for a void function
	FUNCDEF (
		const char* p_name,		//!< Function name
		void (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		type = STYPE_BProcedure;
		func = p_func;
		}
////#endif

	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for a function returning double
	//! \deprecated Use the version that doesn't take createdate, modifydate
	//! or description parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or descriptor parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		double (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		int  p_createdate,
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, p_bInWin32);
		type = STYPE_BFunction;
		func = (VOIDFUNC)p_func;
		}
	#endif

	//! Construct an SML::FUNCDEF for a function returning double
	FUNCDEF (
		const char* p_name,		//!< Function name
		double (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		type = STYPE_BFunction;
		func = (VOIDFUNC)p_func;
		}

	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for a function returning an SML::STRING,
	//! The string returned should be allocated by SML::CONTEXT::StringAlloc()
	//! \deprecated Use the version that doesn't take createdate, modifydate
	//! or description parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or description parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		STRING* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		int  p_createdate,
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, p_bInWin32);
		type = STYPE_BSFunction;
		func = (VOIDFUNC)p_func;
		}
	#endif

	//! Construct an SML::FUNCDEF for a function returning an SML::STRING.
	//! The string returned should be allocated by SML::CONTEXT::StringAlloc()
	FUNCDEF (
		const char* p_name,		//!< Function name
		STRING* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		PARM * p_parms,	//!< Parameter list (null if none needed)
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		type = STYPE_BSFunction;
		func = (VOIDFUNC)p_func;
		}

	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for a function returning an SML::CLASSHASH
	//! \deprecated Use the version that doesn't take createdate, modifydate
	//! or description parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or description parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		CLASSHASH* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		STYPE p_type,	//!< STYPE_ClassHashFunc or STYPE_NumHashFunc
		PARM * p_parms,	//!< Parameter list (null if none needed)
		const char *p_classname,	//!< Name of class held in hash (NULL if STYPE_NumHashFunc)
		int  p_createdate,
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool p_bInWin32 = true	//!< Pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, p_bInWin32);
		type = p_type;
		func = (VOIDFUNC)p_func;
		ClassName = p_classname;
		}
	#endif

	//! Construct an SML::FUNCDEF for a function returning an SML::CLASSHASH
	FUNCDEF (
		const char* p_name,		//!< Function name
		CLASSHASH* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		STYPE p_type,	//!< STYPE_ClassHashFunc or STYPE_NumHashFunc
		PARM * p_parms,	//!< Parameter list (null if none needed)
		const char *p_classname,	//!< Name of class held in hash (NULL if STYPE_NumHashFunc)
		bool p_bInWin32 = true	//!< Pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		type = p_type;
		func = (VOIDFUNC)p_func;
		ClassName = p_classname;
		}

	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for a function returning a class
	//! \deprecated Use the version that doesn't take createdate, modifydate
	//! or description parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or description parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		void* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		STYPE p_type,	//!< STYPE_ClassCopyFunc if return value should be freed by SML when it's done with it.  STYPE_ClassRefFunc if return value should not be freed.
		PARM * p_parms,	//!< Parameter list (null if none needed)
		const char *p_classname,	//!< Name of class returned.  Use SMLCLASSNAME macros from sml.h if possible (or define them if they're not there)
		int  p_createdate,
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, p_bInWin32);
		type = p_type;
		func = (VOIDFUNC)p_func;
		ClassName = p_classname;
		}
	#endif

	//! Construct an SML::FUNCDEF for a function returning a class
	FUNCDEF (
		const char* p_name,		//!< Function name
		void* (* p_func)(int numargs, ARG *args, UINT8 *argtypes, CONTEXT *context),		//!< Actual function
		STYPE p_type,	//!< STYPE_ClassCopyFunc if return value should be freed by SML when it's done with it.  STYPE_ClassRefFunc if return value should not be freed.
		PARM * p_parms,	//!< Parameter list (null if none needed)
		const char *p_classname,	//!< Name of class returned.  Use SMLCLASSNAME macros from sml.h if possible (or define them if they're not there)
		bool p_bInWin32 = true	//!< Pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		type = p_type;
		func = (VOIDFUNC)p_func;
		ClassName = p_classname;
		}

	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for an arbitrary C or C++ function
	//! See the documentation of SML::AUTOCLASS_NOALLOC_T::AddMethod for details
	//! \deprecated Use the version that doesn't take createdate or modifydate
	//! parameters
	DEPRECATED_MSG("Use the version that doesn't take createdate, modifydate, or description parameters")
	FUNCDEF (
		const char* p_name,		//!< Function name
		VOIDFUNC p_func,			//!< Function to call.  Actual function should match the signature specified by the p_signature parameter, but must be cast to (VOIDFUNC).  SML Runtime will take care of the magic.
		SMLFUNCSIG p_signature,	//!< function signature (see SMLFUNCSIG)
		PARM *p_parms,			//!< Parameters (NULL if none needed)
		int p_createdate,
		int p_modifydate = 0,
		const char *p_ClassName = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, 0, p_bInWin32);
		SMLFUNCSIG retsig = FUNCSIG_GetRetType(p_signature);

		if (retsig == SIG_v) type = STYPE_BProcedure;
		else if (retsig == SIG_pv) type = STYPE_ClassRefFunc;
		else if (retsig == SIG_pva) type = STYPE_ClassCopyFunc;
		else if (retsig == SIG_U || retsig == SIG_cU || retsig == SIG_c || retsig == SIG_cc) type = STYPE_BSFunction;
		else type = STYPE_BFunction;

		func = (VOIDFUNC)p_func;
		ClassName = p_ClassName;

		signature = p_signature;
		flags |= SMLFUNC_UseSignature;
		if (!p_bInWin32) flags |= SMLFUNC_NotInWin32;
		}
	#endif

	//! Construct an SML::FUNCDEF for an arbitrary C or C++ function
	//! See the documentation of SML::AUTOCLASS_NOALLOC_T::AddMethod for details
	FUNCDEF (
		const char* p_name,		//!< Function name
		VOIDFUNC p_func,			//!< Function to call.  Actual function should match the signature specified by the p_signature parameter, but must be cast to (VOIDFUNC).  SML Runtime will take care of the magic.
		SMLFUNCSIG p_signature,	//!< function signature (see SMLFUNCSIG)
		PARM *p_parms,			//!< Parameters (NULL if none needed)
		const char *p_ClassName = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);
		SMLFUNCSIG retsig = FUNCSIG_GetRetType(p_signature);

		if (retsig == SIG_v) type = STYPE_BProcedure;
		else if (retsig == SIG_pv) type = STYPE_ClassRefFunc;
		else if (retsig == SIG_pva) type = STYPE_ClassCopyFunc;
		else if (retsig == SIG_U || retsig == SIG_cU || retsig == SIG_c || retsig == SIG_cc) type = STYPE_BSFunction;
		else type = STYPE_BFunction;

		func = (VOIDFUNC)p_func;
		ClassName = p_ClassName;

		signature = p_signature;
		flags |= SMLFUNC_UseSignature;
		if (!p_bInWin32) flags |= SMLFUNC_NotInWin32;
		}

	
	#ifndef NO_DEPRECATED
	//! Construct an SML::FUNCDEF for an arbitrary C or C++ function
	//! See the documentation of SML::AUTOCLASS_NOALLOC_T::AddMethod for details
	//! \deprecated Use SML::AUTOCLASS templates instead
	DEPRECATED_MSG("Use SML::AUTOCLASS templates instead")
	FUNCDEF (
		const char* p_name,		//!< Function name
		METHODPTR p_method,
		SMLFUNCSIG p_signature,
		PARM *p_parms,
		int p_createdate,
		int p_modifydate = 0,
		const char *p_ClassName = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, 0, p_bInWin32);

		SMLFUNCSIG retsig = FUNCSIG_GetRetType(p_signature);

		if (retsig == SIG_v) type = STYPE_BProcedure;
		else if (retsig == SIG_pv) type = STYPE_ClassRefFunc;
		else if (retsig == SIG_pva) type = STYPE_ClassCopyFunc;
		else if (retsig == SIG_U || retsig == SIG_cU || retsig == SIG_c || retsig == SIG_cc) type = STYPE_BSFunction;
		else type = STYPE_BFunction;

		func = 0;
		method = p_method;
		ClassName = p_ClassName;
		signature = p_signature;
		flags |= SMLFUNC_UseSignature | SMLFUNC_IsMemberFunc;
		}
	#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
	// Don't document this one.  It's used internally by the SML::AUTOCLASS
	// methods.
	FUNCDEF (
		const char* p_name,		//!< Function name
		METHODPTR p_method,
		SMLFUNCSIG p_signature,
		PARM *p_parms,
		const char *p_ClassName = 0,
		bool p_bInWin32 = true	//!< pass false if function not available in WIN32_NATIVE builds
		) {
		Init(p_name, p_parms, 0, 0, 0, p_bInWin32);

		SMLFUNCSIG retsig = FUNCSIG_GetRetType(p_signature);

		if (retsig == SIG_v) type = STYPE_BProcedure;
		else if (retsig == SIG_pv) type = STYPE_ClassRefFunc;
		else if (retsig == SIG_pva) type = STYPE_ClassCopyFunc;
		else if (retsig == SIG_U || retsig == SIG_cU || retsig == SIG_c || retsig == SIG_cc) type = STYPE_BSFunction;
		else type = STYPE_BFunction;

		func = 0;
		method = p_method;
		ClassName = p_ClassName;
		signature = p_signature;
		flags |= SMLFUNC_UseSignature | SMLFUNC_IsMemberFunc;
		}
#endif

	#ifndef NO_DEPRECATED
	//! Use this one in an \#ifdef for things which cannot be installed in
	//! the current compile environment (like WIN32_NATIVE)
	//! \deprecated Use SML::AUTOCLASS templates instead
	DEPRECATED_MSG("Use SML::AUTOCLASS templates instead")
	FUNCDEF (
		const char* p_name,		//!< Function name
		SMLFUNCSIG p_signature,
		PARM *p_parms,
		int p_createdate,
		int p_modifydate = 0,
		const char *p_ClassName = 0
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, 0, false);

		SMLFUNCSIG retsig = FUNCSIG_GetRetType(p_signature);

		if (retsig == SIG_v) type = STYPE_BProcedure;
		else if (retsig == SIG_pv) type = STYPE_ClassRefFunc;
		else if (retsig == SIG_pva) type = STYPE_ClassCopyFunc;
		else if (retsig == SIG_U || retsig == SIG_cU || retsig == SIG_c || retsig == SIG_cc) type = STYPE_BSFunction;
		else type = STYPE_BFunction;

		func = 0;
		method = 0;
		ClassName = p_ClassName;
		signature = p_signature;
		flags |= SMLFUNC_UseSignature | SMLFUNC_IsMemberFunc;
		}
	#endif

	#ifndef NO_DEPRECATED
	//! Use this one in an \#ifdef for things which cannot be installed in
	//! the current compile environment (like WIN32_NATIVE)
	//! \deprecated Use SML::AUTOCLASS templates instead
	DEPRECATED_MSG("Use SML::AUTOCLASS templates instead")
	FUNCDEF (
		const char* p_name,
		STYPE p_type,
		PARM * p_parms,
		const char *p_classname,
		int  p_createdate,
		int  p_modifydate = 0,
		const char * p_description = 0,
		bool bInWin32Native = false
		) {
		Init(p_name, p_parms, p_createdate, p_modifydate, p_description, bInWin32Native);
		ClassName = p_classname;
		type = p_type;
		func = 0;
		}
	#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:

	void Init (
		const char* p_name,
		PARM * p_parms,
		int  p_createdate,
		int  p_modifydate,
		const char * p_description,
		bool p_bInWin32
		) {
		name = p_name;
		func = 0;
		parms = p_parms;
		ClassName = 0;
		CreateDate = p_createdate;
		ModDate = p_modifydate;
		minparms = 0;
		maxparms = 0;
		description = p_description;
		sizeofSMLFUNC = sizeof(FUNC);
		sizeofSMLPARM = sizeof(PARM);
		flags = 0;
		signature = SIG_OldStyle;
		method = 0;
		m_funcgroup = 0;
		if (!p_bInWin32) flags |= SMLFUNC_NotInWin32;
		}
#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // end of SML namespace

typedef SML::FUNC SMLFUNC;
typedef SML::FUNCDEF SMLFUNCDEF;

//! Despite the name, these macros are older than SML::FUNCDEF, which is the
//! new-and-improved way of defining these things
#define SMLCLASSFUNCDEF2(func) { #func, func##_FuncType, (VOIDFUNC)func, func##_Parms, func##_RetVal, func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), 0, SIG_OldStyle, 0, 0 }


#define SMLFUNCDEF2(func)      { #func, func##_FuncType, (VOIDFUNC)func, func##_Parms, NULL,          func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), 0, SIG_OldStyle, 0, 0 }

#ifdef WIN32_NATIVE
#define SMLCLASSFUNCDEF_NOWIN32_NATIVE(func) {#func, func##_FuncType, NULL, func##_Parms, func##_RetVal, func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), SMLFUNC_NotInWin32, SIG_OldStyle, 0, 0}
#define SMLFUNCDEF_NOWIN32_NATIVE(func)      {#func, func##_FuncType, NULL, func##_Parms, NULL,          func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), SMLFUNC_NotInWin32, SIG_OldStyle, 0, 0}
#else
#define SMLCLASSFUNCDEF_NOWIN32_NATIVE(func)	{#func, func##_FuncType, (VOIDFUNC)func, func##_Parms, func##_RetVal, func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), SMLFUNC_NotInWin32, SIG_OldStyle, 0, 0}
#define SMLFUNCDEF_NOWIN32_NATIVE(func)		{#func, func##_FuncType, (VOIDFUNC)func, func##_Parms, NULL,          func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), SMLFUNC_NotInWin32, SIG_OldStyle, 0, 0}
#endif


#if !defined(NO_DEPRECATED)
#ifdef WIN32
#pragma deprecated(SMLCLASSFUNCDEF)
#endif
#endif

/**
 * Special version for when we want to define an SML function with the same name
 * as a C function, or with a name that is illegal for a C function (such as
 * left$)  We just name it whatever we want to and add the following
 *
 * 		\#define sml_printf_RealName "printf"
 */
#define SMLFUNCDEF3(func) {func##_RealName, func##_FuncType, (VOIDFUNC)func, func##_Parms, NULL, func##_CreateDate, func##_ModDate,0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), 0, SIG_OldStyle, 0, 0}

#define SMLCLASSFUNCDEF3(func) { func##_RealName, func##_FuncType, (VOIDFUNC)func, func##_Parms, func##_RetVal, func##_CreateDate, func##_ModDate, 0,0, NULL, sizeof(SML::FUNC), sizeof(SML::PARM), 0, SIG_OldStyle, 0, 0 }


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_FUNCTION_H


