/**
 * \file <spatmod/imagetargetbil.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_BIL definition
 *
 * \if NODOC
 * $Id: imagetargetbil.h_v 1.1 2011/08/02 17:55:06 mju Exp $
 *
 * $Log: imagetargetbil.h_v $
 * Revision 1.1  2011/08/02 17:55:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETBIL_H
#define	INC_SPATMOD_IMAGETARGETBIL_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef	INC_MI32_FILERAW_H
#include <mi32/fileraw.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! General band-interleaved (BIL) file target.
//! Can be derived from to include additional headers/trailers.
//! Supports writing non-sequentially and adding to existing file.
class TARGET_BIL : public TARGET {
	public:

		//! Construct with specified input stage and path to target file.
		//! Target does not take ownership of input stage.
		TARGET_BIL (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath					//!< Path to target file
			);

		//! Destructor.
		virtual ~TARGET_BIL ();

		//! Get path to output file.
		const FILEPATH& GetFilePath (
			) const { return (m_filepath); }

		//! Set band spacing bytes.
		//! Contents of file data is undefined, override v_PushSubSection to write.
		//! Must be done before Initialize.
		void SetBandSpacingBytes (
			UINT32 BandSpacingBytes		//!< Spacing between each band within row
			) { m_BandSpacingBytes = BandSpacingBytes; }

		//! Set file header bytes.
		//! Contents of file data is undefined, override v_PushBegin to write.
		//! Must be done before Initialize.
		void SetFileHeaderBytes (
			UINT32 FileHeaderBytes		//!< Spacing before first row in file
			) { m_FileHeaderBytes = FileHeaderBytes; }

		//! Set file trailer bytes.
		//! Contents of file data is undefined, override v_PushEnd to write.
		//! Must be done before Initialize.
		void SetFileTrailerBytes (
			UINT32 FileTrailerBytes		//!< Spacing after last row in file
			) { m_FileTrailerBytes = FileTrailerBytes; }

		//! Set row spacing bytes.
		//! Contents of file data is undefined, override v_PushSubSection to write.
		//! Must be done before Initialize.
		void SetRowSpacingBytes (
			UINT32 RowSpacingBytes		//!< Spacing between rows in file
			) { m_RowSpacingBytes = RowSpacingBytes; }

	protected:

		//! Get file.
		FILE_RAW& GetFile ()
			{ return (m_file); }

		//! Get (non-const) path to output file.
		FILEPATH& GetFilePath (
			) { return (m_filepath); }

		//! Initialize target, does not create file.
		virtual ERRVALUE v_Initialize ();

		//! If override, must call base method to create file.
		//! File will be resized to accommodate entire image including header, spacing and trailer.
		//! Initial file contents are undefined.
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);

		//! If override, must call base method to write data.
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		//! Closes output file.
		virtual ERRVALUE v_PushEnd ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FILEPATH m_filepath;
		FILE_RAW m_file;
		UINT32 m_FileHeaderBytes;		// Space before first row
		UINT32 m_RowSpacingBytes;		// Space between rows
		UINT32 m_BandSpacingBytes;		// Space between bands within each row
		UINT32 m_FileTrailerBytes;		// Space after last row
		UINT32 m_RowTotalBytes;
		UINT32 *m_BandBytes;
		UINT32 *m_BandOffset;

		// Unimplemented
		TARGET_BIL (const TARGET_BIL&);
		TARGET_BIL& operator= (const TARGET_BIL&);

		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETBIL_H
